-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Controls
	description: "Abstract dialog element (control)"
	interface: abstract

deferred class DIALOG_ELEMENT

feature -- Parent
	
	has_dialog_box: BOOLEAN is
			-- Does the object has a valid corresponding dialog?
		deferred
		end

	dialog_box: DIALOG is
			-- The object which uses dialog elements.
		require
			has_dialog: has_dialog_box
		deferred
		ensure
			valid: Result /= Void
		end

feature -- Device independent positioning

	indep_size: RECTANGLE is
			-- Size of element using device independent coordinates.
		deferred
		ensure
			is_copy: -- can be modified.
		end

	indep_set_size (r: RECTANGLE) is
			-- Set element's size in device independent coordinates.
		require
			valid: r /= Void
		deferred
		ensure
			done: r.is_equal (indep_size)
		end

	indep_set_point (p: POINT) is
			-- Set upper-left point of element using device independent units.
		require
			valid_point: p /= Void
		local
			rect: RECTANGLE
		do
			rect := indep_size
			rect.translate_lower (p)
			
			indep_set_size (rect)
		end

	indep_set_width (wi: INTEGER) is
			-- Set element's width using device independent units.
		require
			valid: wi >= 0
		local
			rect: RECTANGLE
			p: POINT
		do
			!!p.make
			p.set_xy (indep_size.lower.x, indep_size.lower.y)
			
			!!rect.make
			rect.translate_lower (p)
			rect.set_width (wi)
			rect.set_height (indep_size.height)

			indep_set_size (rect)
		end

	indep_set_height (he: INTEGER) is
			-- Set element's height using device independent units.
		require
			valid: he >= 0
		local
			rect: RECTANGLE
			p: POINT
		do
			!!p.make 
			p.set_xy (indep_size.lower.x, indep_size.lower.y)
			
			!!rect.make
			rect.translate_lower (p)
			rect.set_width (indep_size.width)
			rect.set_height (he)

			indep_set_size (rect)
		end

feature -- Relative positioning

	place_right (other: DIALOG_ELEMENT) is
			-- Place the current element to the right of `other'.
		require
			valid_other: other /= Void
		local
			r: RECTANGLE
			p: POINT
		do
			r := indep_size
			!!p.make_coord (other.indep_size.upper.x + default_interspace, other.indep_size.lower.y)
			r.translate_lower (p)
			indep_set_size (r)
		end

	place_left (other: DIALOG_ELEMENT) is
			-- Place the current element to the left of `other'.
		require
			valid_other: other /= Void
		local
			r: RECTANGLE
			p: POINT
		do
			r := indep_size
			!!p.make_coord (other.indep_size.lower.x - r.width - default_interspace, other.indep_size.lower.y)
			r.translate_lower (p)
			indep_set_size (r)
		end

	place_above (other: DIALOG_ELEMENT) is
			-- Place the current element above `other'.
		require
			valid_other: other /= Void
		local
			r: RECTANGLE
			p: POINT
		do
			r := indep_size
			!!p.make_coord (other.indep_size.lower.x, other.indep_size.lower.y - r.height - default_interspace)
			r.translate_lower (p)
			indep_set_size (r)
		end
	
	place_under (other: DIALOG_ELEMENT) is
			-- Place the current element under `other'.
		require
			valid_other: other /= Void
		local
			r: RECTANGLE
			p: POINT
		do
			r := indep_size
			!!p.make_coord (other.indep_size.lower.x, other.indep_size.upper.y + default_interspace)
			r.translate_lower (p)
			indep_set_size (r)
		end

	place_just_under (other: DIALOG_ELEMENT) is
			-- Place the current element exactly under 'other'.
		local
			r: RECTANGLE
			p: POINT
		do
			r := indep_size
			!!p.make_coord (other.indep_size.lower.x, other.indep_size.upper.y+1)
			r.translate_lower (p)
			indep_set_size (r)
		end

	same_width (other: DIALOG_ELEMENT) is
			-- Set the current element width to `other''s.
		require
			valid_other: other /= Void
		do
			indep_set_width (other.indep_size.width)
		end
			
	same_height (other: DIALOG_ELEMENT) is
			-- Set the current element height to be the same as `other''s.
		require
			valid_other: other /= Void
		do
			indep_set_height (other.indep_size.height)
		end

	align_left (other: DIALOG_ELEMENT) is
			-- Align left side of this element with the 
			-- left side of `other'.
		require
			ok: other /= Void
		local
			r: RECTANGLE
			newx: INTEGER
		do
			r := indep_size
			newx := other.indep_size.lower.x
			
			if r.upper.x < newx then
				r.set_upper_x (newx)
			end
			r.set_lower_x (newx)
			indep_set_size (r)
		end

	align_right (other: DIALOG_ELEMENT) is
			-- Align the right side of this element with the right 
			-- side of `other'.
		require
			ok: other /= Void
		local
			r: RECTANGLE
			newx: INTEGER
		do
			r := indep_size
			newx := other.indep_size.upper.x

			if r.lower.x > newx then
				r.set_lower_x (newx)
			end
			r.set_upper_x (newx)
			indep_set_size (r)
		end

	align_top (other: DIALOG_ELEMENT) is
			-- Align the top of this element with top of `other'.
		require
			ok: other /= Void
		local
			r: RECTANGLE
			newy: INTEGER
		do
			r := indep_size
			newy := other.indep_size.lower.y
			
			if r.upper.y < newy then
				r.set_upper_y (newy)
			end
			r.set_lower_y (newy)

			indep_set_size (r)
			
		end

	align_bottom (other: DIALOG_ELEMENT) is
			-- Align the bottom of this element with bottom of `other'.
		require
			ok: other /= Void
		local
			r: RECTANGLE
			newy: INTEGER
		do
			r := indep_size
			newy := other.indep_size.upper.y
			
			if r.lower.y > newy then
				r.set_lower_y (newy)
			end
			r.set_upper_y (newy)
			
			indep_set_size (r)
		end

feature {NONE}
	
	default_interspace: INTEGER is 
		deferred
		end

end -- class DIALOG_ELEMENT
