-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Controls
	description: "List box"
	interface: abstract, inheritance

deferred class LISTBOX[G -> LISTBOX_ITEM]

inherit
	DIALOG_ELEMENT
		end

feature -- Manage associated list

	add_list (lst: P_LIST[G]) is
			-- Add items to the list box.
		require
			valid_list: lst /= Void
		local
			it: P_ITERATOR[G]
		do
			-- iterate
			from
				it := lst.iterator
				it.first
			variant
				it.forward_variant
			until
				it.outside
			loop
				add_item (it.item)
				it.forth
			end
		end

	add_item (x: G) is
			-- Add a single item to the list box.
		require
			valid_item: x.is_valid
		deferred
		end

	remove_all is
			-- Remove all items.
		deferred
		end

feature -- Selection

	has_selection: BOOLEAN is
			-- Is (at least) one item selected?
		deferred
		end

	selected_item: G is
			-- Current selected item.
		require
			select_ok: has_selection
		deferred
		end

	has_item (i: G): BOOLEAN is
			-- Is `item' an item of this list?
		deferred
		end

	select_item (newitem: G) is
			-- Select item.
		require
			has: has_item (newitem)
		deferred
		end

feature {NONE}
	
	select_command: GUI_COMMAND
	execute_command: GUI_COMMAND

feature -- Commands

	set_select_command (gc: GUI_COMMAND) is
			-- Set when_selected command.
		require
			nothing: -- Void means no command
		do
			select_command := gc
		ensure
			keep_reference: select_command = gc
		end

	set_execute_command (gc: GUI_COMMAND) is
			-- Set when_execute command.
		require
			nothing: -- Void means no command
		do
			execute_command := gc
		ensure
			keep_reference: execute_command = gc
		end

feature {PUBLIC_NONE} -- Events

	when_selected is
			-- Listbox item 'selected_item' has been selected.
			-- Default: use command.
		require
			selected: has_selection
		do
			if select_command /= Void then
				select_command.execute
			end
		end

	when_executed is
			-- Action to be executed on listbox item 'selected_item'.
			-- Default: use command. 
		require
			selected: has_selection
		do
			if execute_command /= Void then
				execute_command.execute
			end
		end

end -- class LISTBOX
