-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Controls
	description: "Abstract push button"
	interface: abstract, inheritance

deferred class PUSH_BUTTON

inherit
	BUTTON
		end

feature -- Status
	
	is_down: BOOLEAN is
			-- Is the button down?
		deferred
		end

	set_down (newstat: BOOLEAN) is
			-- Set the button 'pushed' state.
		deferred
		ensure
			set: is_down = newstat
		end

feature -- Positioning

	center is
			-- Place button at the centre of the dialog (as 
			-- it would be when optimally arranged).
		require
			db: has_dialog_box
		local
			r: RECTANGLE
			p: POINT
		do
			r := indep_size
			!!p.make_coord ((dialog_box.arranged_size.width - r.width) // 2, r.lower.y)
			r.translate_lower (p)
			indep_set_size (r)
		end

	left_of_center is
			-- Place button at the left of the centre of the 
			-- dialog (as it would be when optimally arranged).
		require
			db: has_dialog_box
		local
			r: RECTANGLE
			p: POINT
		do
			r := indep_size
			!!p.make_coord ((dialog_box.arranged_size.width - default_interspace) // 2, r.upper.y)
			r.translate_upper (p)
			indep_set_size (r)
		end

	right_of_center is
			-- Place button at the right of the centre of the 
			-- dialog (as it would be when optimaly arranged).
		require
			db: has_dialog_box
		local
			r: RECTANGLE
			p: POINT
		do
			r := indep_size
			!!p.make_coord ((dialog_box.arranged_size.width + default_interspace) // 2, r.lower.y)
			r.translate_lower (p)
			indep_set_size (r)
		end
	
feature {PUBLIC_NONE} -- Dialog keyboard interface

	get_default is
			-- Push button is getting default status.
			-- (disable default button having status)
		local
			ok: PUSH_BUTTON
		do
			if has_dialog_box then 
				ok := dialog_box.ok_push_button
				if ok /= Void and then ok.is_default then
					ok.set_default (false)
				end
			end
		end

	lose_default is
			-- Push button has lost default status.
			-- (enable default button having status)
		local
			ok: PUSH_BUTTON
		do
			if has_dialog_box then
				ok := dialog_box.ok_push_button
				if ok /= Void then
					ok.set_default (true)
				end
			end
		end

feature -- Default status

	is_default: BOOLEAN is
			-- Is the button the default push button? (thick border)
		deferred
		end

	set_default (db: BOOLEAN) is
			-- Set the push button to default status.
		deferred
		ensure
			done: db = is_default
		end

feature -- Command

	execute is
			-- Simulate button selection.
		do
			when_executed
		end

end -- class PUSH_BUTTON
