-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Controls
	description: "Abstract scrollbar"
	interface: abstract, inheritance

deferred class SCROLLBAR

feature -- Type of scrollbar

	is_horizontal: BOOLEAN is
			-- Is this scrollbar horizontal?
		deferred
		ensure
			coherent: is_vertical = not is_horizontal
		end

	is_vertical: BOOLEAN is
			-- Is this scrollbar vertical?
		do
			Result := not is_horizontal
		ensure
			coherent: is_vertical = not is_horizontal
		end

feature -- Range & conversion from/to ratios

	Default_range: INTEGER is 100

	range: INTEGER is
			-- Slider position range.
		deferred
		end

	set_range (rg: INTEGER) is
			-- Set slider position range.
		deferred
		ensure
			ok: -- range = rg unless recursive size change
		end

	is_valid_range (r: INTEGER): BOOLEAN is
			-- Is the given range valid?
		do
			Result := (r >= 1 and r <= range)
		ensure
			done: -- Result implies (r >= 1 and Result <= range)
		end

	units_to_ratio (unit: INTEGER): DOUBLE is
			-- Convert from 1-<range>(INTEGER) to 0-1 (DOUBLE)
		require
			valid_range: is_valid_range (unit)
		do
			check range_divisible: range > 1 end
			Result := (unit - 1) / (range - 1)
		ensure
			good_ratio: Result >= 0 and then Result <= 1
		end

	ratio_to_units (ratio: DOUBLE): INTEGER is
			-- Convert from 0-1 (DOUBLE) to 1-<range>(INTEGER).
		require
			good_ratio: ratio >= 0 and then ratio <= 1
		local
			sys: P_SYSTEM
		do
			!!sys
			Result := sys.floor (ratio * (range - 1)) + 1
		ensure
			valid_range: is_valid_range (Result)
		end
			
feature -- Steppings

	line_step: INTEGER
			-- Step to go when line up/down.

	page_step: INTEGER
			-- Step to go when page up/down.

	set_line_step (rg: INTEGER) is
			-- Set line_step.
		require
			positive: rg >= 0
		do
			line_step := rg
		end

	set_page_step (rg: INTEGER) is
			-- Set page_step.
		require
			positive: rg >= 0
		do
			page_step := rg
		end

feature -- Slider position and size

	slider_size: INTEGER is
			-- Size of the slider, on a 1-range scale. 
			-- Note: some implementations only support square sliders.
		deferred
		end

	set_slider_size (s: INTEGER) is
			-- Set the size of the slider on a 1-range scale.
			-- Note: some implementations only support square sliders.
		require
			correct_size: is_valid_range (s)
		deferred
		ensure
			size_set: slider_size = s
		end


	slider_position: INTEGER is
			-- Position of the slider on a 0-range scale.
		deferred
		ensure
			is_valid_range (Result)
		end

	set_slider_position (pos: INTEGER) is
			-- Set the position of the slider on a 1-range scale.
		require
			correct_pos: is_valid_range (pos)
		deferred
		ensure
			pos_set: pos = slider_position
		end

feature -- Event support

	tracked: BOOLEAN is
			-- Is the mouse cursor currently moving slider.
		deferred
		end

feature {NONE}
	
	command: GUI_COMMAND

feature -- Command

	set_command (gc: GUI_COMMAND) is
			-- Set when_slider_moved command.
		require
			nothing: -- Void is no command
		do
			command := gc
		end

feature {PUBLIC_NONE} -- Event(s)

	when_slider_moved (required_position: INTEGER) is
			-- The slider has been moved to the 'requested_position'. The old 
			-- position can be retrieved from 'slider_position'.
			-- Default: move the slider to the requested position, and then 
			-- call 'command'.
		require
			valid_range: is_valid_range (required_position)
		do
			set_slider_position (required_position)
			if command /= Void then
				command.execute
			end
		end

invariant
	valid_dir: is_horizontal xor is_vertical
	
end -- class SCROLLBAR
