-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Controls
	description: "System-managed control"
	interface: inheritance

deferred class STD_CONTROL

inherit
	CONTROL
		redefine
			register, set_font
		end
	COMMON_NOTIFIER
		end

feature {CREATION_NONE} -- Creation

	make (p: WINDOW) is
			-- Abstract control creation.
		require
			valid_parent: p /= Void
		deferred
		end

feature {PUBLIC_NONE} -- registration
	
	register, std_control_register is
			-- Register control.
		do
			-- init (CONTROL)
			control_register -- Precursor

			-- introduce ourselves to notifier
			notifier.register_std_control (Current)
			
			-- subclass the event queue for keyboard & destruction
			wrt.win_subclass_control (self.handle)
		end


feature -- Control's font
	
	set_font (ft: FONT) is
			-- Set control font.
		do
			-- free old
			if control_winfont /= Void then
				control_winfont.force_dispose
			end

			control_winfont := ft.to_windows (Void)

			wrt.win_setfont (self.handle, control_winfont.handle)
		end

feature {NONE}
	
	control_winfont: WIN_FONT

feature {NOTIFIER}
	
	-- Standard control keyboard events. If the control does 
	-- not process a control keyboard event, when_enter, etc 
	-- (see CONTROL) it must set std_control_processed_key 
	-- to false so that the original (OS) control procedure 
	-- of the standard control receives the event.
	
	frozen std_control_processed_key: BOOLEAN
			-- This value is set to true if during the last call to 
			-- {control_}when_character_entered or 
			-- {control_}when_key_pressed the event has actually been 
			-- processed.
			
	frozen std_control_when_key (c: CHARACTER): BOOLEAN is
			-- Standard control keyboard interface. This is 
			-- handled differently than in the plain CONTROL 
			-- because we need to exactly override the place(s) 
			-- where the built-in controls manage the keyboard 
			-- interface. If control_processed_key is true at 
			-- the end of this function, the built-in control 
			-- will not receive the original keyboard event.
			-- cf: control subclassing code in the C runtime.
		local
			vkey: VIRTUAL_KEY
		do
			std_control_processed_key := false

			inspect
				c
			when '%R' then -- Return
				std_control_processed_key := true
				when_enter
			when '%/27/' then -- Escape
				std_control_processed_key := true
				when_escape
			when '%/1/' then -- Up (special runtime value)
				std_control_processed_key := true
				when_up
			when '%/2/' then -- Down (special runtime value)
				std_control_processed_key := true
				when_down
			when '%T' then -- Tab
				std_control_processed_key := true
				!!vkey.make
				vkey.set_shift

				if vkey.is_key_down then
					when_shift_tab
				else
					when_tab
				end
			else 
				-- nothing
			end
			Result := std_control_processed_key
		end

	frozen std_control_when_altkey (c: CHARACTER) is
		do
			when_hotkey (c)
		end

	std_control_when_command (notification: INTEGER) is
			-- Command processing, redefined by actual standard controls.
		require
			redefined_by_heir: false
		do
		end

	std_control_when_scroll (code: INTEGER ; position: INTEGER) is
			-- scroll callback
		require
			redefined_by_heir: false
		do
		end

	std_control_when_focus is
			-- focus given (default do nothing, redefined by push buttons)
		do
		end

	std_control_when_focus_lost is
			-- focus lost.
		do
		end

end -- class STD_CONTROL
