-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Controls
	description: "Built-in drop-down list box"
	interface: client, inheritance

class STD_DROPDOWN_LISTBOX[G -> LISTBOX_ITEM]

inherit
	STD_CONTROL
		redefine
			std_control_when_command, register,
			indep_set_size, default_size
		end
	LISTBOX[G]
		end

creation
	make, make_unsorted

feature {CREATION_NONE} -- Creation

	make (p: WINDOW) is
			-- Create sorted drop-down list box.
		do
			!!self.make_handle (p, wrt.win_combo_create (p.handle, false, true))
			register
		end

	make_unsorted (p: WINDOW) is
			-- Create unsorted drop-down list box.
		require
			valid: p /= Void
		do
			!!self.make_handle (p, wrt.win_combo_create (p.handle, false, false))
			register
		end
	
feature {NONE} -- init

	register, std_dropdown_listbox_register is
			-- Register.
		do
			!!items.make
			set_popup_list_height (Default_popup_list_height)
			std_control_register -- Precursor
		end

feature {NONE} -- Size

	Default_popup_list_height: INTEGER is 100
			-- Default popup list height

	internal_set_size (rect: RECTANGLE) is
			-- New set size that can't change height as it is set by the OS.
		require
			valid: rect /= Void
		do
			wrt.win_resize (self.handle, rect.lower.x, rect.lower.y, rect.upper.x,
					rect.lower.y + popup_list_height) -- popup list height
		ensure
			height_unchanged: self.size.height = rect.height -- windows sets (closed) control size
		end

feature -- Size.

	indep_set_size (rect: RECTANGLE) is
			-- Set listbox size in independent coordinates.
			-- Note: Control's height is set by the operating system, 
			-- so the height is ignored.
		local
			newr: RECTANGLE
			p: POINT
		do
			-- save copy
			!!isize.make
			isize.copy (rect)

			!!p.make
			p.set_xy (dunits.x_indep_to_abs (rect.lower.x),
				dunits.y_indep_to_abs (rect.lower.y))

			!!newr.make
			newr.translate_lower (p)
			newr.set_width (dunits.x_indep_to_abs (rect.width))
			newr.set_height (self.size.height)

			internal_set_size (newr)
		ensure then
			height_unchanged: -- height ignored
		end

	set_popup_list_height (h: INTEGER) is
			-- Set the height of the popup list
		require
			valid_height: h > 0
		do
			popup_list_height := h
			internal_set_size (self.size)
		end

	default_size is
			-- Set size to default.
		local
			p: POINT
		do
			!!p.make
			p.set_x (default_interspace)
			p.set_y (default_interspace)
			indep_set_point (p)
			indep_set_width (default_width)
		end

feature {NONE} -- list
	
	popup_list_height: INTEGER
	items: P_DICTIONARY[G, STRING]

feature -- Items setup

	add_item (item: G) is
			-- Add item to the list.
		do
			wrt.win_combo_laddtext (self.handle, wrt.ew_string (item.name).to_c)
			if items.count = 0 then
				wrt.win_msg_setint (self.handle, wrt.Cb_setcursel, 0)
			end
			items.add (item, item.name)
		end

	remove_all is
			-- Reset list.
		do
			wrt.win_msg_void (self.handle, wrt.Cb_resetcontent)
			!!items.make
		ensure then
			empty: items.is_empty = true
		end

feature {NONE} -- get current item

	get_curitem_idx: INTEGER is
		do
			Result := wrt.win_msg_int (self.handle, wrt.Cb_getcursel)
		end

	get_curitem (idx: INTEGER): G is
		local
			str: STRING
		do
			if idx >= 0 then
				wrt.win_combo_lgettext (self.handle, idx)
				str := wrt.win_laststring

				items.key_search (str) --? dupes handling
				if items.found then
					Result := items.item
				end
			end
		end

feature -- selection

	has_selection: BOOLEAN is
			-- Is an item selected?
		do
			Result := (get_curitem_idx >= 0)
		end

	selected_item: G is
			-- Currently selected item.
		do
			Result := get_curitem (get_curitem_idx)
		end

	has_item (itm: G): BOOLEAN is
			-- Is `itm' in our list?
		local
			it: P_ITERATOR[G]
		do
			from
				it := items.to_item_list.iterator
				it.first
			variant
				it.forward_variant
			until
				it.outside
			loop
				if it.item = itm then
					Result := true
					it.stop
				else
					it.forth
				end
			end
		end

	select_item (new: G) is
			-- Mark `new' as selected.
		do
			wrt.win_combo_selstring (self.handle, wrt.ew_string (new.name).to_c) --? duplicates
			check done: (selected_item.name).is_equal (new.name) end
		end

feature {NONE}
	
	std_control_when_command (not_code: INTEGER) is
		require else
			true
		do
			if not_code = wrt.Cbn_selchange then
				when_selected
			elseif not_code = wrt.Cbn_closeup and has_selection then
				when_executed
			elseif not_code = wrt.Cbn_errspace then
				wrt.raise_exception (wrt.Excep_combo)
			-- do nothing for unknown code
			end
		end

invariant
	list_init: items /= Void

end -- class STD_DROPDOWN_LISTBOX
