-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Controls
	description: "Built-in edit box"
	interface: client, inheritance

class STD_STRINGBOX

inherit
	STD_CONTROL
		redefine
			default_height,
			std_control_when_command,
			when_down, when_up
		end
	STRINGBOX
		end
	CLIPBOARD_OBJECT
		redefine
			cut_selection
		end

creation
	make

feature {CREATION_NONE} -- Creation

	make (p: WINDOW) is
			-- Create edit box.
		do
			!!self.make_handle (p, wrt.win_editbox_create (p.handle, false, false))
			register
		end
			
feature {PUBLIC_NONE} -- Default size

	default_height: INTEGER is
		do
			Result := 12
		end

feature -- Editable string

	contents: STRING is
			-- Edited string.
		do
			Result := self.text -- window's text is the editable text
		end

	set_contents (s: STRING) is
			-- Change editable string.
		do
			self.set_text (s)
		end

feature {PUBLIC_NONE} -- Dialog keyboard interface.

	when_down is
			-- Down arrow managed by control.
		do
			std_control_processed_key := false
		end

	when_up is
			-- Up arrow managed by control.
		do
			std_control_processed_key := false
		end

feature {NONE}

	std_control_when_command (not_code: INTEGER) is
		require else
			redefined: true
		do
			if not_code = wrt.En_change then -- process En_update?
				when_changed
			elseif not_code = wrt.En_errspace then
				wrt.raise_exception (wrt.Excep_editbox)
			end
		end

feature -- Clipboard object

	has_selection: BOOLEAN is
			-- Current clipboard selection.
		do
			Result := wrt.win_msg_bool (self.handle, wrt.Em_getsel)
		end

	copy_selection is
			-- Copy current selection.
		do
			wrt.win_postmsg_void (self.handle, wrt.Wm_copy)
		end

	cut_selection is
			-- Cut selection.
		do
			wrt.win_postmsg_void (self.handle, wrt.Wm_cut)
		end

	delete_selection is
			-- Delete selection.
		do
			wrt.win_postmsg_void (self.handle, wrt.Wm_clear)
		end
	
	paste_clipboard is
			-- Paste clipboard to object.
		do
			wrt.win_postmsg_void (self.handle, wrt.Wm_paste)
		end

	select_all is
			-- Select everything.
		do
		end
			
end -- class STD_STRINGBOX
