-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Controls
	description: "Built-in text box control"
	interface: client

class STD_TEXTBOX
	
inherit
	STD_STRINGBOX
		rename
			contents as editstring,
			set_contents as set_editstring
		redefine
			make, when_enter
		end
	TEXTBOX
		end

creation
	make, make_wrapping

feature {CREATION_NONE} -- Creation

	make (p: WINDOW) is
			-- Create multiline edit control (text editor), no word wrap.
		do
			!!self.make_handle (p, wrt.win_editbox_create (p.handle, true, false))
			register
		end

	make_wrapping (p: WINDOW) is
			-- Create word wrapping multiline edit control.
		require
			valid: p /= Void
		do
			!!self.make_handle (p, wrt.win_editbox_create (p.handle, true, true))
			is_wrapped := true
			register
		end

feature {NONE} -- Keyboard interface event

	when_enter is
			-- Use ENTER in the control.
		do
			std_control_processed_key := false
		end

feature -- Actions

	is_undo_possible: BOOLEAN is
			-- Is there something that can be undone?
		do
			Result := wrt.win_msg_bool (self.handle, wrt.Em_canundo)
		end

	undo is
			-- Undo last user action.
		require
			undoable: is_undo_possible
		do
			wrt.win_msg_void (self.handle, wrt.Em_undo)
		end

	is_wrapped: BOOLEAN
			-- Is text being wrapped?

	modified: BOOLEAN is
			-- Has the text been modified (since last call to `set_contents'
			-- or `reset_modified').
		do
			Result := wrt.win_msg_bool (self.handle, wrt.Em_getmodify)
		end

	reset_modified is
			-- Reset `modified' flag.
		do
			wrt.win_msg_setbool (self.handle, wrt.Em_setmodify, false)
		ensure
			reset: modified = false
		end

feature -- Contents

	contents: P_LIST[STRING] is
			-- Text.
		local
			wsl: WIN_STRING_LIST
		do
			!!wsl
			Result := wsl.string_to_list (editstring)
		end


	set_contents (c: P_LIST[STRING]) is
			-- Change editable text.
		local
			wsl: WIN_STRING_LIST
		do
			!!wsl
			set_editstring (wsl.list_to_string (c))
		end
		
end -- class STD_TEXTBOX
