-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "Capabilities of a graphic device"
	interface: client

class DEVICE_CAPABILITIES

inherit
	WIN_API
		end

creation {GRAPHIC_DEVICE}
	make

feature {NONE} -- Creation

	make (gd: GRAPHIC_DEVICE) is
		do
			parent := gd
		end

feature {NONE}
	
	parent: GRAPHIC_DEVICE

feature -- Precondition
	
	is_device_ready: BOOLEAN is
			-- Is the parent device valid?
		do
			Result := parent.is_ready
		end

feature -- Device type

	is_plotter: BOOLEAN is
			-- Is the device a plotter?
		require
			ready: is_device_ready
		do
			Result := (wrt.win_gdi_capint (parent.handle, wrt.Technology) = wrt.Dt_plotter)
		end

	is_raster_display: BOOLEAN is
			-- Is the device a raster display?
		require
			ready: is_device_ready
		do
			Result := (wrt.win_gdi_capint (parent.handle, wrt.Technology) = wrt.Dt_rasdisplay)
		end

	is_raster_printer: BOOLEAN is
			-- Is the device a raster printer?
		require
			ready: is_device_ready
		do
			Result := (wrt.win_gdi_capint (parent.handle, wrt.Technology) = wrt.Dt_rasprinter)
		end

	is_raster_camera: BOOLEAN is
			-- Is the device a raster camera?
		require
			ready: is_device_ready
		do
			Result := (wrt.win_gdi_capint (parent.handle, wrt.Technology) = wrt.Dt_rascamera)
		end

	is_character_stream: BOOLEAN is 
			-- Is the device a character stream?
		require
			ready: is_device_ready
		do
			Result := (wrt.win_gdi_capint (parent.handle, wrt.Technology) = wrt.Dt_charstream)
		end

	is_metafile: BOOLEAN is
			-- Is the device a metafile?
		require
			ready: is_device_ready
		do
			Result := (wrt.win_gdi_capint (parent.handle, wrt.Technology) = wrt.Dt_metafile)
		end

	is_display_file: BOOLEAN is
			-- Is the device a display file?
		require
			ready: is_device_ready
		do
			Result := (wrt.win_gdi_capint (parent.handle, wrt.Technology) = wrt.Dt_dispfile)
		end

feature -- Device size and definition
	
	width_mm: INTEGER is
			-- Width of the device screen (millimetres).
		require
			ready: is_device_ready
		do
			Result := wrt.win_gdi_capint (parent.handle, wrt.Horzsize)
		end
	
	height_mm: INTEGER is
			-- Height of the device screen (millimetres).
		require
			ready: is_device_ready
		do
			Result := wrt.win_gdi_capint (parent.handle, wrt.Vertsize)
		end

	width_pixel: INTEGER is
			-- Width of the device screen (pixels).
		require
			ready: is_device_ready
		do
			Result := wrt.win_gdi_capint (parent.handle, wrt.Horzres)
		end

	height_pixel: INTEGER is
			-- Height of the device screen (pixels).
		require
			ready: is_device_ready
		do
			Result := wrt.win_gdi_capint (parent.handle, wrt.Vertres)
		end

	width_dpi: INTEGER is
			-- Number of pixel per inch (dot-per-inch) along the horizontal axis.
		require
			ready: is_device_ready
		do
			Result := wrt.win_gdi_capint (parent.handle, wrt.Logpixelsx)
		end
	
	height_dpi: INTEGER is
			-- Number of pixel per inch (dot-per-inch) along the vertical axis.
		require
			ready: is_device_ready
		do
			Result := wrt.win_gdi_capint (parent.handle, wrt.Logpixelsy)
		end

	screen_pixel: RECTANGLE is
			-- Helper function providing rectangle of screen in pixels for 
			-- window positioning, etc.
		require
			ready: is_device_ready
		do
			!!Result.make
			Result.set_width (width_pixel)
			Result.set_height (height_pixel)
		ensure
			is_copy: Result /= Void
		end

feature -- Colors

	color_bits_per_pixel: INTEGER is
			-- Number of adjacent colour bits for each pixel.
		require
			ready: is_device_ready
		do
			Result := wrt.win_gdi_capint (parent.handle, wrt.Bitspixel)
		end

	color_plane: INTEGER is
			-- Number of colour planes.
		require
			ready: is_device_ready
		do
			Result := wrt.win_gdi_capint (parent.handle, wrt.Planes)
		end

	color_number: INTEGER is
			-- Number of colours currently available (typically 20 on screens).
		require
			ready: is_device_ready
		do
			Result := wrt.win_gdi_capint (parent.handle, wrt.Numcolors)
		end

	has_palette: BOOLEAN is
			-- Is display palettised (or true colour)?
		require
			ready: is_device_ready
		do
			Result := wrt.win_gdi_capbool (parent.handle, wrt.Rastercaps, wrt.Rc_palette)
		end

	palette_bits_per_pixel: INTEGER is
			-- Actual colour resolution of a palettised device in bits per pixel.
		require
			ready: is_device_ready
			palette: has_palette
		do
			Result := wrt.win_gdi_capint (parent.handle, wrt.Colorres)
		end

	palette_reserved: INTEGER is
			-- Number of reserved entries in the system palette.
		require
			ready: is_device_ready
			palette: has_palette
		do
			Result := wrt.win_gdi_capint (parent.handle, wrt.Numreserved)
		end

	palette_size: INTEGER is
			-- Nymber of entries in the system palette.
		require
			ready: is_device_ready
			palette: has_palette
		do
			Result := wrt.win_gdi_capint (parent.handle, wrt.Sizepalette)
		end

feature -- Raster capabilities

	has_raster_bitmap: BOOLEAN is
			-- Has raster capabilities? (pixel, bitmap transfer)
		require
			ready: is_device_ready
		do
			Result := wrt.win_gdi_capbool (parent.handle, wrt.Rastercaps, wrt.Rc_bitblt)
		end

feature {DRAWABLE} -- Drawing capabilities

	has_ellipse: BOOLEAN is
		do
			Result := wrt.win_gdi_capbool (parent.handle, wrt.Curvecaps, wrt.Cc_ellipses)
		end

	has_pie: BOOLEAN is
		do
			Result := wrt.win_gdi_capbool (parent.handle, wrt.Curvecaps, wrt.Cc_pie)
		end

	has_chord: BOOLEAN is
		do
			Result := wrt.win_gdi_capbool (parent.handle, wrt.Curvecaps, wrt.Cc_chord)
		end

	has_arc: BOOLEAN is
		do
			Result := has_chord
		end

	has_polyline: BOOLEAN is
		do
			Result := wrt.win_gdi_capbool (parent.handle, wrt.Linecaps, wrt.Lc_polyline)
		end

	has_polygon_alternate: BOOLEAN is
		do
			Result := wrt.win_gdi_capbool (parent.handle, wrt.Polygonalcaps, wrt.Pc_polygon)
		end

	has_polygon_winding: BOOLEAN is
		do
			Result := wrt.win_gdi_capbool (parent.handle, wrt.Polygonalcaps, wrt.Pc_windpolygon)
		end

	has_rectangle: BOOLEAN is
		do
			Result := wrt.win_gdi_capbool (parent.handle, wrt.Polygonalcaps, wrt.Pc_rectangle)
		end

	has_round_rectangle: BOOLEAN is
		do
			Result := wrt.win_gdi_capbool (parent.handle, wrt.Curvecaps, wrt.Cc_roundrect)
		end

feature -- Text capabilities

	can_rotate_text: BOOLEAN is
			-- Can the device rotate text?
		require
			ready: is_device_ready
		do
			Result := wrt.win_gdi_capbool (parent.handle, wrt.Textcaps, wrt.Tc_cr_any)
		end

	can_rotate_text_90: BOOLEAN is
			-- Can the device rotate text at 90, 180, 270 degrees?
		require
			ready: is_device_ready
		do
			Result := wrt.win_gdi_capbool (parent.handle, wrt.Textcaps, wrt.Tc_cr_90)
		ensure
			implication: can_rotate_text
		end

invariant
	device_exist: parent /= Void

end -- class DEVICE_CAPABILITIES
