-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "Drawable multiple-segment line"
	comment: "Polyline will be drawn up to but not including the last point"
	interface: client

class DRAWABLE_POLYLINE

inherit
	DRAWABLE
		end

creation
	make

feature {CREATION_NONE} -- Creation
	
	make is
			-- Create.
		do
			!!line.make
		end

feature {NONE} -- Internal position
	
	line: P_LINKED_LIST[POINT]
			-- Internal list of points.

feature -- Position

	empty: BOOLEAN is
			-- Is the polyline empty (no points)?
		do
			Result := line.is_empty
		end

	reset is
			-- Reset the point list representing the polyline.
		do
			!!line.make
		ensure
			done: empty
		end
	
	add_point (pt: POINT) is
			-- Add a new point at the end of the polyline.
		require
			valid_point: pt /= Void
		do
			add_line (pt)
		ensure
			ok: not empty
		end

feature {GRAPHIC_DEVICE}
	
	draw (gd: GRAPHIC_DEVICE) is
		local
			aout: F_MEMORY
		do
			aout := to_points_array
			wrt.win_gdi_polyline (gd.handle, aout.to_windows, aout.size//4)
		end

feature {NONE}
	
	add_line (pt: POINT) is
		local
			newpt: POINT
		do
			!!newpt.make
			newpt.copy (pt)
			line.add (newpt)
		end

feature {NONE}
	
	to_points_array: F_MEMORY is
		local
			i: INTEGER
			it: P_ITERATOR[POINT]
		do
			!!Result.make (line.count * 8) -- count*2 (double_word)

			from
				i := 1
				it := line.iterator
				it.first
			variant
				it.forward_variant
			until
				it.outside
			loop
				Result.set_double_word (i, it.item.x)
				i := i + 4
				Result.set_double_word (i, it.item.y)
				i := i + 4

				it.forth
			end
		end

invariant
	valid_position: line /= Void

end -- class DRAWABLE_POLYLINE
