-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "Draw unicode text in an 8-bit context"
	interface: client, tower

class DRAWABLE_UNICODE_TEXT

inherit
	DRAWABLE_TEXT
		redefine
			internal_get_size, internal_text_out,
			internal_ext_text_out
		end

creation
	make

feature {NONE}
	
	charmap: F_MEMORY

feature
	
	set_character_map (map: ARRAY[INTEGER]) is
			-- Set the character map used to convert 8-bit characters to 
			-- UNICODE. map.item (input_char.code) is the Unicode character
			-- code. This must be set before any drawing takes place.
			-- (this setting is not changed when `reset' is called).
		require
			ok: map /= Void
			map_size: map.lower = 0 and map.upper = 255 -- 0x00 -> 0xFF
			char_size: -- for_all i, 0 <= map.item (i) <= 65535
		local
			i, idx: INTEGER
		do
			!!charmap.make (map.count * 4)
			from
				i := map.lower
				idx := 1
			variant map.upper + 1 - i until
				i > map.upper
			loop
				charmap.set_double_word (idx, map.item (i))
				idx := idx + 4
				i := i + 1
			end
		end

feature {NONE} -- (Internal) text functions

	internal_get_size (dc: GRAPHIC_DEVICE) is
		do
			check has_charmap: charmap /= Void end

			wrt.win_gdi_wtextsize (charmap.to_windows, dc.handle, wrt.ew_string (text).to_c)
		end

	internal_text_out (dc: GRAPHIC_DEVICE) is
		do
			check has_charmap: charmap /= Void end

			wrt.win_gdi_wtext (charmap.to_windows,
				dc.handle, pos_x, pos_y, wrt.ew_string (text).to_c)
		end

	internal_ext_text_out (dc: GRAPHIC_DEVICE) is
		do
			check has_charmap: charmap /= Void end

			wrt.win_gdi_wextext (charmap.to_windows,
					dc.handle, pos_x, pos_y, area_x, area_y, area_xp, area_yp,
					false, true, wrt.ew_string (text).to_c)
		end

invariant
	narrow_characters: -- CHARACTER.code <= 255

end -- class DRAWABLE_UNICODE_TEXT
	
