-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "(Enhanced) metafile device"
	interface: client

class METAFILE_DEVICE

inherit
	GRAPHIC_DEVICE
		end

creation
	make_file, make_memory

feature {CREATION_NONE} -- Creation

	make_file (refdev: GRAPHIC_DEVICE; width, height: INTEGER; file: STRING) is
			-- Create (enhanced) metafile named `file', from 
			-- exisiting device, fitting in a rectangle of 
			-- `width' and `height' (0.01 mm units).
		require
			valid_reference: refdev /= Void and then refdev.is_ready
			valid_size: width > 0 and height > 0
			valid_file: file /= Void
		do
			initialise
			set_handle (wrt.win_gdi_emf (wrt.ew_string (file).to_c, refdev.handle, width, height))
			if is_ready then
				initialise_attributes
			end
		ensure
			done: -- is_ready if success.
		end

	make_memory (refdev: WINDOW_DEVICE; width, height: INTEGER) is
			-- Create a memory metafile.
		require
			valid_reference: refdev /= Void and then refdev.is_ready
			valid_size: width > 0 and height > 0
		do
			initialise
			set_handle (wrt.win_gdi_emf (wrt.ew_string ("").to_c, refdev.handle, width, height))
			check ok: is_ready end
			initialise_attributes
		ensure
			done: -- is_ready if success.
		end

feature -- Access

	is_ready: BOOLEAN is
			-- Is the device ready?
		do
			Result := is_valid_handle
		end

	prepare is
			-- Device is ready at creation.
		do
		ensure then
			ready_at_creation: false
		end

	release is
			-- Release the device (close metafile).
		do
			dispose
		end

feature {NONE}
	
	dispose is
			-- Finalisation.
		local
			hmet: INTEGER
		do
			if handle /= Invalid_handle then
				hmet := win_gdi_closeemf (handle)
				if hmet /= 0 then
					win_gdi_delemf (hmet)
				end

				handle := Invalid_handle
			end
		end

feature {CLIPBOARD_METAFILE}
	
	get_metafile is
		do
			last_metafile := wrt.win_gdi_closeemf (handle)
			reset_handle
		end

	delete_metafile is
		do
			if last_metafile /= 0 then
				wrt.win_gdi_delemf (last_metafile)
			end
			last_metafile := 0
		end
	
	last_metafile: INTEGER

end -- class METAFILE_DEVICE
