-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "Simple page mapping modes"
	interface: client

class PAGE_MAPPING

inherit
	GRAPHIC_MAPPING
		redefine
			copy, is_equal
		end

creation
	make

feature {CREATION_NONE} -- Creation

	make is
		do
			!!origin.make
			text
		end

feature {PAGE_MAPPING} -- Attribute(s)
	
	mode: INTEGER
	origin: POINT
	
feature {WIN_API}
	
	duplicate: GRAPHIC_MAPPING is
			-- Duplicate current.
		do
			!PAGE_MAPPING!Result.make
			Result.copy (Current)
		end

feature -- General

	copy (other: like Current) is
		do
			mode := other.mode
			origin.copy (other.origin)
		end

	is_equal (other: like Current): BOOLEAN is
		do
			Result := (mode = other.mode) and origin.is_equal (other.origin)
		end

feature -- Modes

	is_cartesian: BOOLEAN is
			-- Is the current mapping mode cartesian?
			-- Cartesian means x coordinates grow rightward, y when 
			-- downwards, and that circles are round and squares square
			-- (the device is isotropic).
		do
			Result := not (mode = wrt.Mm_text)
		end

	text is
			-- Pixel (device) coordinates; x to the right, y downwards
		do
			mode := wrt.Mm_text
		end

	low_metric is
			-- 0.1 millimetre, cartesian.
		do
			mode := wrt.Mm_lometric
		end
	
	high_metric is
			-- 0.01 millimetre, cartesian.
		do
			mode := wrt.Mm_himetric
		end

	low_english is
			-- 0.01 inch, cartesian.
		do
			mode := wrt.Mm_lometric
		end

	high_english is
			-- 0.001 inch, cartesian.
		do
			mode := wrt.Mm_himetric
		end

	twip is
			-- 1/20th of a point (1/1440th of an inch), cartesian.
		do
			mode := wrt.Mm_twips
		end

feature -- Origin

	set_origin (pt: POINT) is
			-- Set where the physical device origin (0, 0) is relatively to 
			-- the new logical origin of the device (in logical coordinates).
		require
			valid_point: pt /= Void
		do
			origin.copy (pt)
		end

feature {GRAPHIC_DEVICE}
	
	set_mode (device: GRAPHIC_DEVICE) is
		do
			-- use default Gm_compatible mode
			wrt.win_gdi_mapmode (device.handle, mode)
			wrt.win_gdi_origin (device.handle, origin.x, origin.y)
		end

end -- class PAGE_MAPPING
