-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "Holds the coordinates of a point in a 2D space"
	interface: client

class POINT

inherit
	P_TEXT_OBJECT
		redefine
			out
		end

creation
	make, make_coord,
	from_string

feature {CREATION_NONE} -- Creation

	make is
			-- Create.
		do
			x := 0
			y := 0
		ensure
			ok: is_origin
		end

	make_coord (ex, ey: INTEGER) is
			-- Create from coordinates.
		do
			x := ex
			y := ey
		ensure
			done: x = ex and y = ey
		end

feature
	
	out: STRING is
			-- String representation.
		do
			!!Result.make (0)
			Result.append_character ('(')
			Result.append_string (x.out)
			Result.append_character (',')
			Result.append_string (y.out)
			Result.append_character (')')
		end

feature {NONE} -- Effect P_TEXT_OBJECT
	
	object_to_string (tm: P_TEXT_OBJECT_MANAGER) is
		do
			tm.put_integer (1) -- Placeholder for version
			tm.put_integer (x)
			tm.put_integer (y)
		end

	string_to_object (tm: P_TEXT_OBJECT_MANAGER) is
		local
			version: INTEGER
		do
			tm.get_integer  version := tm.last_integer
			tm.get_integer  x := tm.last_integer
			tm.get_integer  y := tm.last_integer

			if not (tm.in_success and then version = 1) then make end
		end

feature -- Actual coordinates

	x, y: INTEGER

feature -- Set point coordinates

	set_x (xc: INTEGER) is
			-- Set the horizontal coordinate.
		do
			x := xc
		end

	set_y (yc: INTEGER) is
			-- Set the vertical coordinate.
		do
			y := yc
		end

feature -- Helper procedures

	set_xy (cx, cy: INTEGER) is
			-- Set both coordinates.
		do
			set_x (cx)
			set_y (cy)
		end

feature -- Point operations

	is_origin: BOOLEAN is
			-- Is the point the origin (0, 0)?
		do
			Result := (x = 0) and then (y = 0)
		end

	is_less_horizontally (other: POINT): BOOLEAN is
		do
			Result := x <= other.x
		ensure
			done: Result = (x <= other.x)
		end

	is_less_vertically (other: POINT): BOOLEAN is
		do
			Result := y <= other.y
		ensure
			done: Result = (y <= other.y)
		end

	is_less (other: POINT): BOOLEAN is
		do
			Result:= is_less_horizontally (other) and then is_less_vertically (other)
		ensure
			done: Result = (is_less_horizontally (other) and then is_less_vertically (other))
		end

end -- class POINT
