-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "RGB colour specification (dithered on palletised devices)"
	interface: client

class RGB_COLOR

inherit
	COLOR
		end
	P_TEXT_OBJECT
		end

creation
	white, black,
	dark_red, dark_green, dark_yellow, dark_blue, dark_magenta, dark_cyan, 
	money_green, sky_blue, cream, 
	light_gray, light_grey, medium_gray, medium_grey, dark_gray, dark_grey,
	normal_red, normal_green, normal_blue,
	yellow, magenta, cyan,
	rgb, from_string

feature -- Standard colours. Always available (even when palette changed).

	white is
		do
			rgb (255, 255, 255)
		end

	black is
		do
			rgb (0, 0, 0)
		end

	dark_red is
		do
			rgb (128, 0, 0)
		end

	dark_green is
		do
			rgb (0, 128, 0)
		end

	dark_yellow is
		do
			rgb (128, 128, 0)
		end

	dark_blue is
		do
			rgb (0, 0, 128)
		end

	dark_magenta is
		do
			rgb (128, 0, 128)
		end

	dark_cyan is
		do
			rgb (0, 128, 128)
		end

	light_gray, light_grey is
		do
			rgb (192, 192, 192)
		end
	
	money_green is
		do
			rgb (192, 220, 192)
		end

	sky_blue is
		do
			rgb (166, 202, 240)
		end

	cream is
		do
			rgb (255, 251, 240)
		end

	medium_gray, medium_grey is
		do
			rgb (160, 160, 164)
		end

	dark_gray, dark_grey is
		do
			rgb (128, 128, 128)
		end

	normal_red is
		do
			rgb (255, 0, 0)
		end

	normal_green is
		do
			rgb (0, 255, 0)
		end

	yellow is
		do
			rgb (255, 255, 0)
		end

	normal_blue is
		do
			rgb (0, 0, 255)
		end

	magenta is
		do
			rgb (255, 0, 255)
		end

	cyan is
		do
			rgb (0, 255, 255)
		end
	
feature -- RGB

	rgb (r, g, b: INTEGER) is
			-- Set RGB colour components. On palletised displays, the colour will be 
			-- dithered from the standard colours above (e.g. from 20 colours, even on 
			-- a 256 colour palettised display). On the other hand, the colour will not 
			-- change when the palette is changed. To use a wider range of colours a 
			-- palette must be used in association with RGB_PALETTE_COLOR.
		require
			valid_values: -- see invariant
		do
			red := r
			green := g
			blue := b
		ensure
			done: red = r and green = g and blue = b
		end

feature {NONE} -- Serialisation

	object_to_string (tm: P_TEXT_OBJECT_MANAGER) is
		do
			tm.put_integer (1)
			tm.put_integer (red)
			tm.put_integer (green)
			tm.put_integer (blue)
		end
	
	string_to_object (tm: P_TEXT_OBJECT_MANAGER) is
		local
			version: INTEGER
		do
			tm.get_integer version := tm.last_integer
			tm.get_integer red := tm.last_integer
			tm.get_integer green := tm.last_integer
			tm.get_integer blue := tm.last_integer
			if version /= 1 or not tm.in_success then
				rgb (0, 0, 0)
			end
		end

feature -- RGB Components
	
	red, green, blue: INTEGER

feature {WIN_API}
	
	to_windows: INTEGER is
		do
			Result := wrt.win_gdi_rgbcolor (red, green, blue)
		end

invariant
	valid_red: red >= 0 and red <= 255
	valid_green: green >= 0 and green <= 255
	valid_blue: blue >= 0 and blue <= 255

end -- class RGB_COLOR
