-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics
	description: "Text metrics corresponding to the device's current font"
	interface: client

class TEXT_METRICS

inherit
	WIN_API
		end
	COMMON_NOTIFIER
		end

creation {GRAPHIC_DEVICE}
	make

feature {NONE} 
	
	make (device: GRAPHIC_DEVICE) is
		require
			valid: device /= Void
		do
			parent := device
		end

feature {NONE}
	
	parent: GRAPHIC_DEVICE

feature 
	
	is_ready: BOOLEAN is
			-- Is the associated device ready?
		do
			Result := parent.is_ready
		end

feature -- Metrics

	font: LOGICAL_FONT is
			-- Logical font corresponding to the current font.
		require
			ready: is_ready
		do
			!!Result.make_device (parent)
		ensure
			is_copy: Result /= Void -- can be modified.
			no_rotation: Result.character_orientation = 0 and Result.line_orientation = 0
		end

	font_families: P_LIST[LOGICAL_FONT] is
			-- List of all font families available (a single representative 
			-- font is randomly elected for each family).
		require
			ready: is_ready
		do
			Result := internal_font_family ("")
		ensure
			done: Result /= Void
			is_copy: -- list regenerated for each call.
		end

	font_family (typeface_name: STRING): P_LIST[LOGICAL_FONT] is
			-- List all fonts from the `typeface_name' family.
		require
			ready: is_ready
			valid: typeface_name /= Void and then typeface_name.count > 0
		do
			Result := internal_font_family (typeface_name)
		ensure
			done: Result /= Void
			is_copy: -- new list each time.
		end

feature {NONE} -- Family implementation

	Logfont_buffer_size: INTEGER is 25

	internal_font_family (face_name: STRING): P_LINKED_LIST[LOGICAL_FONT] is
			-- Retrieve logical font list for a family name.
		require
			not_void: face_name /= Void
		local
			logfont_buffer: F_MEMORY
			logfont_single: F_MEMORY
			fonts_count, result_count: INTEGER
			a_font: LOGICAL_FONT
			i, start_i: INTEGER
		do
			-- try with a sensible size
			!! logfont_buffer.make (Logfont_buffer_size * wrt.Logfont_size)
			fonts_count := wrt.win_gdi_enumfonts (parent.handle, wrt.ew_string (face_name).to_c,
								Logfont_buffer_size, logfont_buffer.to_windows)
			
			-- try again with full size if not enough
			if fonts_count > Logfont_buffer_size then
				logfont_buffer.close

				!! logfont_buffer.make (fonts_count * wrt.Logfont_size)
				result_count := wrt.win_gdi_enumfonts (parent.handle, wrt.ew_string (face_name).to_c,
								fonts_count, logfont_buffer.to_windows)
				
				check consistent_font_counts: result_count = fonts_count end
				fonts_count := (fonts_count) .min (result_count)
			end

			from
				i := 1
				start_i := 1
				!! Result.make
			variant 
				fonts_count + 1 - i
			until
				i > fonts_count
			loop
				logfont_single := logfont_buffer.subset (start_i, wrt.Logfont_size)
				!! a_font.make_logfont (logfont_single.to_windows)
				logfont_single.close --? fixme, make_memory in LOGICAL_FONT

				Result.add (a_font)

				-- next
				start_i := start_i + wrt.Logfont_size
				i := i + 1
			end

			logfont_buffer.close
		end

feature -- Metrics

	is_vector: BOOLEAN is
			-- Is the current font a vector font?
		require
			ready: is_ready
		do
			Result := wrt.int_to_bool (wrt.win_gdi_textmetr (parent.handle, wrt.Ftm_vector))
		end

	is_truetype: BOOLEAN is
			-- Is the current font a true type font?
		require
			ready: is_ready
		do
			Result := wrt.int_to_bool (wrt.win_gdi_textmetr (parent.handle, wrt.Ftm_truetype))
		end
	
	is_device: BOOLEAN is
			-- Is the current font a device specific font?
		require
			ready: is_ready
		do
			Result := wrt.int_to_bool (wrt.win_gdi_textmetr (parent.handle, wrt.Ftm_device))
		end

	ascent: INTEGER is
			-- Current font's ascent (units above the base line).
		require
			ready: is_ready
		do
			Result := wrt.win_gdi_textmetr (parent.handle, wrt.Ftm_ascent)
		end

	descent: INTEGER is
			-- Current font's descent (units under the base line).
		require
			ready: is_ready
		do
			Result := wrt.win_gdi_textmetr (parent.handle, wrt.Ftm_descent)
		end

	height: INTEGER is
			-- Current font's line height (ascent + descent).
		require
			ready: is_ready
		do
			Result := wrt.win_gdi_textmetr (parent.handle, wrt.Ftm_height)
		end

	internal_leading: INTEGER is
			-- Current font's internal leading (blank space above most characters).
		require
			ready: is_ready
		do
			Result := wrt.win_gdi_textmetr (parent.handle, wrt.Ftm_intleading)
		end

	external_leading: INTEGER is
			-- Current font's external leading (space between row, added to height).
		require
			ready: is_ready
		do
			Result := wrt.win_gdi_textmetr (parent.handle, wrt.Ftm_extleading)
		end

	maximum_character_width: INTEGER is
			-- Current font maximum char width.
		require
			ready: is_ready
		do
			Result := wrt.win_gdi_textmetr (parent.handle, wrt.Ftm_maxcharw)
		end
	
	--? add overhang, first/last/default/break char
	
invariant
	valid_parent: parent /= Void

end -- class TEXT_METRICS
