-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Graphics, Internal
	description: "Low-level logical font data"
	interface: internal

class WIN_LOGFONT

inherit
	WIN_API
		end

feature {NONE} -- LOGFONT definition

	Height_offset: INTEGER is 1 -- (1-size) scale
	Width_offset: INTEGER is 5
	Escapement_offset: INTEGER is 9
	Orientation_offset: INTEGER is 13
	Weight_offset: INTEGER is 17
	Italic_offset: INTEGER is 21
	Underline_offset: INTEGER is 22
	Strikeout_offset: INTEGER is 23
	Charset_offset: INTEGER is 24
	Outprec_offset: INTEGER is 25
	Clipprec_offset: INTEGER is 26
	Quality_offset: INTEGER is 27
	Pitchfamily_offset: INTEGER is 28
	Facename_offset: INTEGER is 29

	Facename_length: INTEGER is 32 -- LF_FACESIZE

	Logfont_size: INTEGER is 60

	Pitch_mask: INTEGER is 15 -- 0x0F 00001111
	Family_mask: INTEGER is 240 -- 0xF0 11110000

feature {NONE}
	
	integer_to_boolean (i: INTEGER): BOOLEAN is
		do
			if i /= 0 then Result := true end
		end

	boolean_to_integer (bool: BOOLEAN): INTEGER is
		do
			if bool = true then Result := 1 end
		end

feature {NONE}
	
	init_logfont is
		do
			!!struct.make (Logfont_size)
		end

feature {NONE}
	
	lf_iface: STRING is
		do
			Result := struct.cstring (Facename_offset, Facename_length)
			--? Unicode
		ensure
			is_copy: Result /= Void
		end

	lf_iheight: INTEGER is
		do
			Result := struct.double_word (Height_offset)
		end

	lf_iwidth: INTEGER is
		do
			Result := struct.double_word (Width_offset)
		end

	lf_escapement: INTEGER is
		do
			Result := struct.double_word (Escapement_offset)
		end

	lf_orientation: INTEGER is
		do
			Result := struct.double_word (Orientation_offset)
		end

	lf_iweight: INTEGER is
		do
			Result := struct.double_word (Weight_offset)
		end

	lf_italic: BOOLEAN is
		do 
			Result := integer_to_boolean (struct.byte (Italic_offset))
		end
		
	lf_underline: BOOLEAN is
		do 
			Result := integer_to_boolean (struct.byte (Underline_offset))
		end
	
	lf_strike_out: BOOLEAN is
		do
			Result := integer_to_boolean (struct.byte (Strikeout_offset))
		end

	lf_charset: INTEGER is
		do
			Result := struct.byte (Charset_offset)
		end

	-- (Extended) outprecision, clipprecision, quality
	
	lf_pitch: INTEGER is
		do 
			Result := struct.bit_and (struct.byte (Pitchfamily_offset), Pitch_mask)
		end

	lf_family: INTEGER is
		do
			Result := struct.bit_and (struct.byte (Pitchfamily_offset), Family_mask)
		end

	lf_set_iface (str: STRING) is
		require
			valid: str /= Void
		do
			struct.set_cstring (Facename_offset, Facename_length, str)
		end

	lf_set_iheight (i: INTEGER) is 
		do 
			struct.set_double_word (Height_offset, i) 
		end

	lf_set_iwidth (i: INTEGER) is 
		do
			struct.set_double_word (Width_offset, i)
		end
	
	lf_set_escapement (i: INTEGER) is 
		do
			struct.set_double_word (Escapement_offset, i)
		end
	
	lf_set_orientation (i: INTEGER) is 
		do
			struct.set_double_word (Orientation_offset, i)
		end

	lf_set_iweight (i: INTEGER) is
		do
			struct.set_double_word (Weight_offset, i)
		end
	
	lf_set_italic (b: BOOLEAN) is 
		do 
			struct.set_byte (Italic_offset, boolean_to_integer (b))
		end
	
	lf_set_underline (b: BOOLEAN) is 
		do
			struct.set_byte (Underline_offset, boolean_to_integer (b)) 
		end

	lf_set_strike_out (b: BOOLEAN) is 
		do 
			struct.set_byte (Strikeout_offset, boolean_to_integer (b))
		end
	
	lf_set_charset (i: INTEGER) is 
		do 
			struct.set_byte (Charset_offset, i)
		end
	
	lf_set_pitch (i: INTEGER) is
		do
			lf_set_pitch_family (i, lf_family)
		end

	lf_set_family (i: INTEGER) is
		do
			lf_set_pitch_family (lf_pitch, i)
		end
	
	lf_set_pitch_family (pit, fam: INTEGER) is 
		do
			struct.set_byte (Pitchfamily_offset, struct.bit_or (pit, fam))
		end
		
feature {WIN_LOGFONT, WIN_API}
	
	struct: F_MEMORY

invariant
	init: struct /= Void

end -- class WIN_LOGFONT

