-- Pylon: a foundation library; version 0.82(beta); 1998-01-04

-- 
-- Copyright (c) 1997 Franck Arnaud, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY FRANCK ARNAUD AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR 
-- ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
-- DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE CODE.
-- 

indexing
	cluster: Pylon, Container
	description: "Fundamental container"
	interface: abstract

deferred class P_CONTAINER[G]

inherit
	ANY
		undefine
			is_equal, copy
		redefine
			out
		end

feature -- Status

	count : INTEGER is
			-- Number of items in the container.
		deferred
		ensure
			positive: Result >= 0
		end

	is_empty : BOOLEAN is
			-- Is the container empty?
		do
			Result := count = 0
		end

	is_readable : BOOLEAN is
			-- Is the container currently readable?
		do
			Result := true
		end

	is_writable : BOOLEAN is
			-- Is the container currently writable?
		deferred 
		end

	
	item : G is
			-- Current item.
		require
			readable: is_readable
			not_empty: not is_empty
		deferred
		end

feature -- Update

	reset is
			-- Remove all objects from the container.
		require
			writable: is_writable
		deferred
		ensure
			done: is_empty
		end

feature -- Output

	out : STRING is
			-- String representation.
		do
			!!Result.make(0)
			Result.append_string("[")
			Result.append_string(count.out)
			Result.append_string("]")
		ensure then
			is_copy: -- new string
		end

feature -- Utility
	
	is_void(x : ANY) : BOOLEAN is
			-- Void assertion for generic parameters.
		do
			Result := (x = Void)
		end

invariant
	empty: is_empty = (count = 0)

end
