-- Pylon: a foundation library; version 0.82(beta); 1998-01-04

-- 
-- Copyright (c) 1997 Franck Arnaud, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY FRANCK ARNAUD AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR 
-- ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
-- DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE CODE.
-- 

indexing
	cluster: Pylon, Container
	description: "Iterator of indexed structure"
	interface: client

class P_SEQUENCE_ITERATOR[G]

inherit
	P_ITERATOR[G]
		end

creation {P_SEQUENCE}
	make

feature {NONE}
	
	container : P_SEQUENCE_LIST[G]
	index : INTEGER
	locked_index : INTEGER

feature {P_SEQUENCE}
	
	make(c:P_SEQUENCE_LIST[G]) is
		require
			valid: c /= Void 
		do
			container := c
			index := 0
		end

feature

	outside : BOOLEAN is
		do
			Result := (index = 0)
		end

	iterable : BOOLEAN is
		do
			Result := not container.is_empty
		end

feature -- Synchronisation
	
	synchronize_container is
		do
			container.set_index(index)
		end

feature -- Iteration
	
	first is
			-- Go to first item of the container.
		do
			if not container.is_empty then
				if outside then
					container.protect
				end
				index := 1
			end
		end

	last is
			-- Go to the last item of the container.
		do
			if not container.is_empty then
				if outside then
					container.protect
				end
				index := container.count
			end
		end

	stop is
			-- Finish.
		do
			index := 0
			container.unprotect
		end

	forth is
			-- Next item.
		do
			if index = container.count then
				stop
			else
				index := index + 1
			end
		end

	back is
			-- Previous item.
		do
			if index = 1 then
				stop
			else
				index := index - 1
			end
		end

	item : G is
			-- Return the item at the current position.
		local
			oldindex : INTEGER
		do
			oldindex := container.index
			container.set_index(index)
			Result := container.item
			container.set_index(oldindex)
		end

	synchronize is
			-- Synch from container.
		do
			check container_ok: not container.is_empty end
			if outside then
				container.protect
			end
			index := container.index
		end

feature
	
	is_locked : BOOLEAN is
		do
			Result := locked_index /= 0
		end

	lock is
		do
			locked_index := index
			index := 0
			
			container.lock(locked_index)
			container.add_lock
			container.unprotect
		end

	unlock is
		do
			container.protect
			container.remove_lock
			container.unlock(locked_index)

			index := locked_index
			locked_index := 0
		end

feature -- Assertions support.

	forward_variant : INTEGER is
		do
			if not outside then
				Result := container.count - index + 1
			end
		end

	backward_variant : INTEGER is
		do
			if not outside then
				Result := index
			end
		end

invariant
	valid_container: container /= Void
	valid_index: index >= 0 and then index <= container.count
	protection: (not outside) implies container.is_protected 
		
end
