-- Pylon: a foundation library; version 0.82(beta); 1998-01-04

-- 
-- Copyright (c) 1997 Franck Arnaud, England, E.U.
-- 
-- Permission is hereby granted, without written agreement and without
-- licence or royalty fees, to use, copy, modify, or distribute this source
-- code or binary programs using it or part of it, subject to the following
-- condition:
-- 
-- The above copyright notice and the following disclaimer must appear in
-- all copies or substantial portions of this source code. (Binary programs
-- derived from the source code can be distributed without restriction or
-- mention of the origin of the source.)
-- 
-- THIS SOFTWARE IS PROVIDED BY FRANCK ARNAUD AND THE AUTHORS ``AS IS'' AND
-- ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-- IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-- ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR 
-- ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
-- DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOURCE CODE.
-- 

indexing
	cluster: Pylon, Iteration
	description: "Linear iterators for forest trees"
	interface: client, internal

deferred class P_TREE_ITERATOR[G]

inherit
	P_ONEWAY_ITERATOR[G]
		end

feature {NONE}
	
	container : P_GENERAL_TREE[G]
	cur_node : P_TREE_NODE[G]
	locked_node : P_TREE_NODE[G]
	varcount : INTEGER
	disp : P_DISPENSER[P_TREE_NODE[G]]

feature {P_TREE} -- Creation

	make(c : like container) is
		do
			container := c
			initialize_iterator
		end

feature -- Access

	outside : BOOLEAN is
		do
			Result := cur_node = Void
		end

	iterable : BOOLEAN is
		do
			Result := not container.is_empty
		end

	item : G is
		do
			Result := cur_node.item
		end

feature
	
	synchronize_container is
		do
			container.set_current_node(cur_node)
		end

feature {NONE}
	
	initialize(node : P_TREE_NODE[G]) is
		require
			valid: node /= Void
		do
			varcount := container.count + 1 -- 1 for init forth
			cur_node := node
			disp.reset
			initialize_iteration(node)
		end
	

feature {NONE} -- internal deferred feartures.

	initialize_iterator is
			-- Create the proper dispenser.
		deferred
		end

	initialize_iteration(root : P_TREE_NODE[G]) is
			-- Initialise iteration.
		require
			valid: root /= Void
		deferred
		end

feature -- Iteration

	first is
		do
			if not container.is_empty then
				if outside then
					container.protect
				end

				initialize(container.root_node)
			end
		end

	synchronize is
			-- Iterate sub tree starting at current node.
		do
			check valid: not container.is_empty end
			if outside then
				container.protect
			end

			initialize(container.current_node)
		end

	stop is
		do
			cur_node := Void
			varcount := 0
			disp.reset
			container.unprotect
		end
	
feature
	
	is_locked : BOOLEAN is
		do
			Result := (locked_node /= Void)
		end

	lock is
		do
			-- swap
			locked_node := cur_node
			cur_node := Void
			varcount := 0

			-- lock
			locked_node.lock
			container.add_lock
			container.unprotect
		end

	unlock is
		do
			-- unlock
			container.protect
			container.remove_lock
			locked_node.unlock
			
			-- swap
			cur_node := locked_node
			locked_node := Void

			initialize(cur_node)
		end

	
feature
	
	forward_variant : INTEGER is
		do
			if not outside then
				Result := varcount
			end
		end
	
end
