/*
 * Fenestra - Constant class generation
 * Copyright (c) 1996 AlterSoft
 *
 * This program is used to generate the constants classes for
 * direct manipulation of OS structures through F_MEMORY.
 *
 * Compile: cl mk_const.c (for Microsoft C)
 */

#include <windows.h>
#include <stdio.h>

#define OFFSET(a,b,c) offset(a,(char *) &b,(char *) &(b.c))

void offset(char *name, char *start, char *end) {
	printf("\t%s : INTEGER is %d\n",name,(int) (end-start) + 1);
}  

void header(char *name) {
	printf("\nclass %s\n\nfeature { NONE } -- Constants\n\n",name);
}

void end(void) {
	printf("\nend\n");
}

void lenght(char *name, int size) {
	printf("\t%s : INTEGER is %d\n",name,size);	
}

void do_systime(void)
{
	SYSTEMTIME st;
	  
    header("WIN_SYSTIME_CONST");

	lenght("Systime_lenght",sizeof(SYSTEMTIME));
    
    OFFSET("Year_offset",st,wYear);
	OFFSET("Month_offset",st,wMonth);
	OFFSET("Dayweek_offset",st,wDayOfWeek);
	OFFSET("Day_offset",st,wDay);
	OFFSET("Hour_offset",st,wHour);
	OFFSET("Minute_offset",st,wMinute);
	OFFSET("Second_offset",st,wSecond);
	OFFSET("Ms_offset",st,wMilliseconds);
	
	end();
}

void do_finddata(void)
{
	WIN32_FIND_DATA fd;

	header("WIN_FINDDATA_CONST");

	lenght("Finddata_lenght",sizeof(WIN32_FIND_DATA));
	
	printf("\n");

	OFFSET("Fileattrib_offset",fd,dwFileAttributes);
	OFFSET("Creation_offset",fd,ftCreationTime);
	OFFSET("Lastaccess_offset",fd,ftLastAccessTime);
	OFFSET("Lastwrite_offset",fd,ftLastWriteTime);
	OFFSET("Sizehigh_offset",fd,nFileSizeHigh);
	OFFSET("Sizelow_offset",fd,nFileSizeLow);
	OFFSET("Filename_offset",fd,cFileName);
	OFFSET("Altname_offset",fd,cAlternateFileName);

	printf("\n");

	lenght("Filename_lenght",MAX_PATH);
	lenght("Altname_lenght",14);

	end();
}

void do_logfont(void)
{
	LOGFONT lf;

	header("WIN_LOGFONT_CONST");

	lenght("Logfont_size",(int) sizeof(LOGFONT));
	lenght("Facename_lenght",(int) LF_FACESIZE);
	
	printf("\n");

	OFFSET("Height_offset",lf,lfHeight);
   	OFFSET("Width_offset",lf,lfWidth);
	OFFSET("Escapement_offset",lf,lfEscapement);
	OFFSET("Orientation_offset",lf,lfOrientation);
	OFFSET("Weight_offset",lf,lfWeight);
	OFFSET("Italic_offset",lf,lfItalic);
	OFFSET("Underline_offset",lf,lfUnderline);
	OFFSET("Strikeout_offset",lf,lfStrikeOut);
	OFFSET("Charset_offset",lf,lfCharSet);
	OFFSET("Outprec_offset",lf,lfOutPrecision);
   	OFFSET("Clipprec_offset",lf,lfClipPrecision);
	OFFSET("Quality_offset",lf,lfQuality);
	OFFSET("Pitchfamily_offset",lf,lfPitchAndFamily);
	OFFSET("Facename_offset",lf,lfFaceName); /* [LF_FACESIZE]; */

	end();
}

void do_accel(void)
{
	ACCEL ka;

	header("WIN_ACCEL_CONST");
	
	lenght("Accel_size",(int) sizeof(ACCEL));
	printf("\n");

	OFFSET("Flags_offset",ka,fVirt);
	OFFSET("Key_offset",ka,key);
	OFFSET("Cmd_offset",ka,cmd);
	
	end();
}

void do_overlapped(void)
{
	OVERLAPPED ov;

	header("WIN_OVERLAPPED_CONST");

	lenght("Overlapped_size",(int) sizeof(OVERLAPPED));
	printf("\n");

	OFFSET("Internal_offset",ov,Internal);
	OFFSET("Internalhigh_offset",ov,InternalHigh);
	OFFSET("Offset_offset",ov,Offset);
	OFFSET("Offsethigh_offset",ov,OffsetHigh);
	OFFSET("Event_offset",ov,hEvent);

	end();
}

void main(int argc, char **argv)
{
	do_logfont();
	do_finddata();
	do_systime();
	do_accel();
	do_overlapped();

	lenght("Paintstruct_size",(int) sizeof(PAINTSTRUCT));
}
