-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Bubble
	description: "Rectangular drawing object"
	interface: inheritance

deferred class BOX

inherit
	BUBBLE_OBJECT
		end

feature {NONE}

	Handle_size: INTEGER is 8
	
	handles: P_LIST[P_PAIR[RECTANGLE,BOX_RESIZE]]

	make_handles is
			-- Create handles.
		local
			pt: POINT
			res: BOX_RESIZE
		do
			!P_LINKED_LIST[P_PAIR[RECTANGLE,BOX_RESIZE]]!handles.make
			!!pt.make

			pt := position.lower
			!!res.make_north_west (position)
			add_handle (pt, res)
			
			pt.set_x (position.center.x)
			!!res.make_north (position)
			add_handle (pt, res)
			
			pt.set_x (position.upper.x)
			!!res.make_north_east (position)
			add_handle (pt, res)

			pt.set_y (position.center.y)
			!!res.make_east (position)
			add_handle (pt, res)

			pt.set_y (position.upper.y)
			!!res.make_south_east (position)
			add_handle (pt, res)

			pt.set_x (position.center.x)
			!!res.make_south (position)
			add_handle (pt, res)

			pt.set_x (position.lower.x)
			!!res.make_south_west (position)
			add_handle (pt, res)

			pt.set_y (position.center.y)
			!!res.make_west (position)
			add_handle (pt, res)
		end

	add_handle (handle_centre: POINT; box_resize: BOX_RESIZE) is
			-- Add single handle.
		local
			rect: RECTANGLE
			hand: P_PAIR[RECTANGLE,BOX_RESIZE]
		do
			!!rect.make
			
			rect.set_width (Handle_size)
			rect.set_height (Handle_size)
			rect.translate_center (handle_centre)

			!!hand
			hand.set_one (rect)
			hand.set_two (box_resize)

			handles.add (hand)
		end

feature -- Size
	
	position: RECTANGLE
			-- Bounding rectangle.

	set_position (sz: RECTANGLE) is
			-- Change size.
		require
			ok: sz /= Void
		do
			position.copy (sz)
			handles := Void
		end

	bounding_box: RECTANGLE is
			-- Bounding box including handles.
		local
			low, up: POINT
			margin: INTEGER
		do
			margin := Handle_size * 2 // 3

			low := position.lower
			low.set_x (low.x - margin)
			low.set_y (low.y - margin)
			up := position.upper
			up.set_x (up.x + margin)
			up.set_y (up.y + margin)

			!!Result.make
			Result.set_points (low, up)
		end

	set_center (centre_point: POINT) is
			-- Translate centre of object.
		require
			ok: centre_point /= Void
		do
			position.translate_center (centre_point)
		end
		

feature -- Moving

	is_inside (pt: POINT): BOOLEAN is
		do
			Result := bounding_box.has_point (pt)
		end

	anchor (pt: POINT): BUBBLE_SHADOW is
		local
			it: P_ITERATOR[P_PAIR[RECTANGLE,BOX_RESIZE]]
		do
			if handles = Void then
				make_handles
			end

			from
				it := handles.iterator
				it.first
			variant it.forward_variant until it.outside loop
				if it.item.one.has_point (pt) then
					Result := it.item.two
					it.stop
				else
					it.forth
				end
			end

			if Result = Void and then position.has_point (pt) then
				!BOX_MOVE!Result.make (position, pt.x-position.lower.x, pt.y-position.lower.y)
			end

			-- init anchor
			if Result /= Void then
				Result.set_anchor (pt)
			end
		end

	resize (bs: BUBBLE_SHADOW) is
		local
			box: BOX_SHADOW
		do
			box ?= bs
			check ok: box /= Void end

			set_position (box.position)
		end

feature -- Drawing

	draw_selection (dev: GRAPHIC_DEVICE) is
		local
			it: P_ITERATOR[P_PAIR[RECTANGLE,BOX_RESIZE]]
			dr: DRAWABLE_RECTANGLE
			gm: GRAPHIC_MODE
			gr: GRAPHICS_ROUTINES
			clr: RGB_COLOR
		do
			!!gr

			-- xor mode
			!!gm.make
			gm.inverse
			dev.set_mode (gm)

			-- frame
			!!clr.black
			dev.set_pen (gr.color_pen (clr))
			dev.set_brush (gr.void_brush)
			
			!!dr.make
			dr.set_position (position)
			dev.draw (dr)

			-- handles
			if handles = Void then
				make_handles
			end

			dev.set_brush (gr.color_brush (clr))
			from
				it := handles.iterator
				it.first
			variant it.forward_variant until it.outside loop
				dr.set_position (it.item.one)
				dev.draw (dr)

				it.forth
			end

			gm.pen
			dev.set_mode (gm)
		end

end
