-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Bubble
	description: "Drawing area"
	interface: client

class BUBBLE_DRAW

inherit
	CHILD_WINDOW_COLOR
		redefine
			when_created, when_drawn, when_resized,
			when_mouse_clicked, when_mouse_rclicked,
			when_horizontal_scrolled, when_vertical_scrolled
		end

creation
	make

feature {NONE}
	
	make (ow: like parent; tb: BUBBLE_TOOLBAR) is
		local
			frame_clr: RGB_COLOR
		do
			toolbar := tb
			!!frame_clr.white
			make_child_scroll_color (ow, frame_clr)
		end

feature {GUI_COMMAND} -- Commands

	current_mode: CMD_TOOL
	toolbar: BUBBLE_TOOLBAR

	tool_edit: CMD_TOOL

feature -- Menu

	tool_menu: POPUP_MENU
	edit_menu: POPUP_MENU
	zoom_menu: POPUP_MENU

feature -- Name

	name: FILE_TITLE

feature
	
	overlapped_parent: OVERLAPPED_WINDOW is
			-- Find overlapped_window parent.
		local
			maxgen: INTEGER -- maximum recursion
			w: WINDOW
		do
			w := Current

			from 
				maxgen := 5 
			variant maxgen until Result /= Void loop
				check no_end: w /= Void end
				w := w.parent
				Result ?= w
				maxgen := maxgen - 1
			end
		ensure
			found: Result /= Void
		end

feature {NONE}
	
	init_menus is
			-- Initialise menus.
		require
			once_only: -- menu created once
		local
			head: HEAD_MENU_ITEM
			item: MENU_ITEM
			cmd: GUI_COMMAND
			ct: CMD_TOOL
		do
			-- Tool
			check once_tool: tool_menu = Void end

			!!tool_menu.make (overlapped_parent)
			tool_menu.set_name ("&Tool")

			!!head.make ("&Selection")
			!CMD_TOOL_EDIT!ct.make (Current, head)
			head.set_command (ct)
			toolbar.add_command (ct)
			tool_menu.add_item (head)

			tool_edit := ct

			!!item.make ("&Bubble")
			item.set_leader (head)
			!CMD_TOOL_BUBBLE!ct.make (Current, item)
			item.set_command (ct)
			toolbar.add_command (ct)
			tool_menu.add_item (item)

			!!item.make ("&Arrow")
			item.set_leader (head)
			!CMD_TOOL_ARROW!ct.make (Current, item)
			item.set_command (ct)
			toolbar.add_command (ct)
			tool_menu.add_item (item)

			!!item.make ("&Frame")
			item.set_leader (head)
			!CMD_TOOL_FRAME!ct.make (Current, item)
			item.set_command (ct)
			toolbar.add_command (ct)
			tool_menu.add_item (item)

			-- Edit
			check once_only: edit_menu = Void end
			
			!!edit_menu.make (overlapped_parent)
			edit_menu.set_name ("&Edit")

			!!item.make ("Cu&t")
			item.set_help ("Cut current selected object to clipboard.")
			edit_menu.add_item (item)
			!CMD_EDIT_CUT!cmd.make (Current)
			item.set_command (cmd)

			!!item.make ("&Copy")
			item.set_help ("Copy current selected object to clipboard.")
			edit_menu.add_item (item)
			!CMD_EDIT_COPY!cmd.make (Current)
			item.set_command (cmd)


			!!item.make ("&Paste")
			item.set_help ("Paste clipboard to drawing.")
			edit_menu.add_item (item)
			!CMD_EDIT_PASTE!cmd.make (Current)
			item.set_command (cmd)

			!!item.make ("&Delete")
			item.set_help ("Delete current selected object.")
			edit_menu.add_item (item)
			!CMD_EDIT_DEL!cmd.make (Current)
			item.set_command (cmd)

			edit_menu.add_separator

			!!item.make ("P&roperties")
			item.set_help ("Edit properties of selected object.")
			edit_menu.add_item (item)
			!CMD_EDIT_PROPERTIES!cmd.make (Current)
			item.set_command (cmd)

			-- zoom
			check once_only: zoom_menu = Void end
			
			!!zoom_menu.make (overlapped_parent)
			zoom_menu.set_name ("&Zoom")

			!!head.make ("100%%")
			head.set_help ("Reset zoom.")
			!CMD_ZOOM!cmd.make (Current, 1.0)
			head.set_command (cmd)
			-- add later

			!!item.make ("25%%")
			item.set_help ("Zoom to 25%%.")
			item.set_leader (head)
			zoom_menu.add_item (item)
			!CMD_ZOOM!cmd.make (Current, 0.25)
			item.set_command (cmd)
			
			!!item.make ("50%%")
			item.set_help ("Zoom to 50%%.")
			item.set_leader (head)
			zoom_menu.add_item (item)
			!CMD_ZOOM!cmd.make (Current, 0.5)
			item.set_command (cmd)

			!!item.make ("75%%")
			item.set_help ("Zoom to 75%%.")
			item.set_leader (head)
			zoom_menu.add_item (item)
			!CMD_ZOOM!cmd.make (Current, 0.75)
			item.set_command (cmd)
			
			-- 100%
			zoom_menu.add_item (head)

			!!item.make ("150%%")
			item.set_leader (head)
			item.set_help ("Zoom to 150%%.")
			zoom_menu.add_item (item)
			!CMD_ZOOM!cmd.make (Current, 1.5)
			item.set_command (cmd)

			!!item.make ("200%%")
			item.set_leader (head)
			item.set_help ("Zoom to 200%%.")
			zoom_menu.add_item (item)
			!CMD_ZOOM!cmd.make (Current, 2.0)
			item.set_command (cmd)

		end

feature {GUI_COMMAND} -- Mode (change)
	
	set_mode (new_mode: CMD_TOOL) is
		require
			ok: new_mode /= Void
		do
			current_mode := new_mode
		ensure
			done: current_mode = new_mode
		end

feature {NONE} -- Mapping attribute(s)

	zoom: DOUBLE

	Picture_width: INTEGER is 2000
	Picture_height: INTEGER is 2000

feature -- Mapping

	set_zoom (nz: DOUBLE) is
			-- Set zoom level.
		require
			not_zero: nz /= 0.0
		do
			if zoom /= nz then
				zoom := nz
				apply_mapping
				when_resized
				paint
			end
		end

feature {NONE} -- Mapping change

	init_mapping is
			-- Initialise mapping
		do
			zoom := 1.0
			horizontal_scrollbar.set_range (Picture_width)
			horizontal_scrollbar.set_slider_position (1)
			vertical_scrollbar.set_range (Picture_height)
			vertical_scrollbar.set_slider_position (1)
			apply_mapping
		end

	apply_mapping is
			-- Change mapping
		local
			cm: CUSTOM_MAPPING
			pt: POINT
		do
			!!pt.make
			pt.set_x (horizontal_scrollbar.slider_position)
			pt.set_y (vertical_scrollbar.slider_position)
			
			!!cm.make
			cm.set_origin (pt)
			cm.set_scale (zoom)
			device.set_mapping (cm)
		end

feature {PUBLIC_NONE} -- Scrolling

	when_horizontal_scrolled (req_pos: INTEGER) is
			-- Scroll horizontally.
		do
			horizontal_scrollbar.set_slider_position (req_pos)
			apply_mapping
			paint
		end

	when_vertical_scrolled (req_pos: INTEGER) is
			-- Vertical scroll.
		do
			vertical_scrollbar.set_slider_position (req_pos)
			apply_mapping
			paint
		end

feature {GUI_COMMAND, MOUSE_HANDLER}

	drw: DRAWING
			-- Actual drawing.
	
	selected: BUBBLE_OBJECT
			-- Selected object (if relevant).
	
	clipboard: BUBBLE_OBJECT;
			-- Internal clipboard.

feature {GUI_COMMAND}
	
	set_clipboard (no: BUBBLE_OBJECT) is
			-- Change clipboard.
		require
			ok: no /= Void
		do
			clipboard := clone (no)
		ensure
			equal: clipboard.is_equal (no)
			not_same: clipboard /= no
		end

feature -- File command(s)

	new is
			-- Reset drawing.
		do
			clipboard := Void
			selected := Void
			drw.reset
			name.reset
		end

feature -- Logical redraw

	paint_logical_rectangle (lr: RECTANGLE) is
			-- Paint rectangle, in logical coordinates.
		local
			nr: RECTANGLE
		do
			!!nr.make
			if device.is_ready then
				nr.set_points (device.logical_to_device (lr.lower),
					device.logical_to_device (lr.upper))
			else
				device.prepare
				nr.set_points (device.logical_to_device (lr.lower),
					device.logical_to_device (lr.upper))
				device.release
			end

			paint_rectangle (nr)
		end

feature {BUBBLE_MOUSE, GUI_COMMAND} -- Selection

	has_selected: BOOLEAN is
			-- Is an object currently selected?
		do
			Result := selected /= Void
		end

	unselect is
			-- Cancel selection.
		do
			if selected /= Void then
				if device.is_ready then
					selected.unselect (device)
				else
					device.prepare
					selected.unselect (device)
					device.release
				end
				selected := Void
			end
		end
	
	select_object (pt: POINT) is
			-- Select object at point.
		local
			candidate: BUBBLE_OBJECT
		do
			-- already selected
			if not (selected /= Void and then selected.bounding_box.has_point (pt)) then
				candidate := drw.find_object (pt)
				
				device.prepare  
				
				-- remove existing selection
				unselect

				-- actual selection
				if candidate /= Void then
					candidate.select_object (device)
				end
				
				device.release
				selected := candidate
			end -- already selected
		end

feature {PUBLIC_NONE}
	
	when_created is
			-- Init.
		do
			set_border (false)

			-- init objects
			!!name.make (overlapped_parent)
			!!drw.make

			-- mode
			init_mapping
			
			-- menu
			init_menus
			tool_edit.execute -- default tool/mode
		end

	when_mouse_clicked (pt: POINT) is
			-- Left mouse button click.
		do
			current_mode.click (pt)
		end

	when_mouse_rclicked (pt: POINT) is
			-- Right mouse button click.
		do
			current_mode.right_click (pt)
		end

	when_resized is
			-- Adapt scrollbars.
		local
			page: POINT
			gr: GRAPHICS_ROUTINES
		do
			!!gr
			
			!!page.make
			page.set_x (client_size.width * 4 // 5)
			page.set_y (client_size.height * 4 // 5)

			device.prepare
			page := gr.device_to_logical_size (device, page)
			device.release

			horizontal_scrollbar.set_page_step (page.x)
			horizontal_scrollbar.set_line_step (page.x//10)
			vertical_scrollbar.set_page_step (page.y)
			vertical_scrollbar.set_line_step (page.y//10)
		end

	when_drawn is
			-- Paint.
		local	
			ft: LOGICAL_FONT
		do
			-- prepare font
			!!ft.make
			ft.set_face_name ("Times New Roman")
			ft.set_character_height (12) --set_point (10)
			device.set_font (ft)

			-- actual drawing
			drw.paint (device)

			-- selection
			if selected /= Void then
				selected.draw_selection (device)
			end

		end

end -- class BUBBLE_DRAW
