-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Bubble
	description: "Mouse handler for BUBBLE_DRAW (edit mode)"
	interface: client

class BUBBLE_MOUSE

inherit
	MOUSE_HANDLER
		redefine
			parent, when_down, when_up,
			when_drag_started, when_dragged, when_drag_finished
		end

creation
	make

feature -- Parent
	
	parent: BUBBLE_DRAW

feature {NONE}
	
	target: BUBBLE_OBJECT
	anchor: BUBBLE_SHADOW

	do_move: BOOLEAN

feature {NONE}
	
	logical_point: POINT is
		local
			dev: GRAPHIC_DEVICE
		do
			dev := parent.device
			if dev.is_ready then
				Result := dev.device_to_logical (point)
			else
				dev.prepare
				Result := dev.device_to_logical (point)
				dev.release
			end
		end

feature {PUBLIC_NONE}
	
	when_down is
		local
			pre_target: BUBBLE_OBJECT
		do
			do_move := false
			anchor := Void
			
			pre_target := parent.selected
			parent.select_object (logical_point)
			target := parent.selected

			if pre_target /= target then
				do_move := true -- don't cancel if new selection
			end
		end

	when_up is
		do
			if target /= Void and do_move = false then
				parent.unselect
			end
		end

	when_drag_started is
		do
			do_move := true
			if target /= Void then
				anchor := target.anchor (logical_point)
				
				if anchor /= Void then
					-- draw outline
					parent.device.prepare
					target.draw_outline (parent.device, anchor)
					parent.device.release
				end
			end
		end
	
	when_dragged is
		do
			if anchor /= Void then
				parent.device.prepare
				
				-- remove previous outline
				target.draw_outline (parent.device, anchor)
				
				-- new outline
				anchor.set_anchor (logical_point)
				target.draw_outline (parent.device, anchor)

				parent.device.release
			end
		end

	when_drag_finished is
		local
			old_rect: RECTANGLE
		do
			if anchor /= Void then
				-- remove outline
				parent.device.prepare
				target.draw_outline (parent.device, anchor)
				parent.device.release
			   
				-- update object
				old_rect := parent.drw.dependent_area (target)
				target.resize (anchor)
				
				parent.paint_logical_rectangle (old_rect)
				parent.paint_logical_rectangle (parent.drw.dependent_area (target))
			end

			-- reset
			anchor := Void
			target := Void
		end

end
