-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Bubble
	description: "Base graphic object"
	interface: inheritance

deferred class BUBBLE_OBJECT

inherit
	P_TEXT_OBJECT
		undefine
			copy, is_equal
		end

feature -- Properties

	properties (dp: BUBBLE_DRAW): MODELESS_DIALOG is
			-- Start property dialog.
		require
			ok: dp /= Void
		deferred
		ensure
			no_properties: -- Void implies not has_properties
		end

feature -- Type information

	type: STRING is
			-- Type name.
		deferred
		end

	is_dependent (other: BUBBLE_OBJECT): BOOLEAN is
			-- Is other depending on this object?
			-- (default: false)
		do
		end

feature -- Selection

	is_inside (pt: POINT): BOOLEAN is
			-- Is the point "inside" the object? (selection)
		require
			ok_pt: pt /= Void
		deferred
		end

	select_object (gd: GRAPHIC_DEVICE) is
			-- Select object.
		require
			ok: gd /= Void
		do
			draw_selection (gd)
		end

	unselect (gd: GRAPHIC_DEVICE) is
			-- Unselect object.
		require
			ok: gd /= Void
		do
			draw_selection (gd) -- XOR, cancel
		end

feature -- Draw selection

	draw_selection (gd: GRAPHIC_DEVICE) is
			-- Draw selection (XOR).
		require
			ok: gd /= Void and then gd.is_ready
		deferred
		end

feature -- Manipulation

	bounding_box: RECTANGLE is
			-- Rectangle enclosing the object (redraws).
		deferred
		ensure
			ok: Result /= Void
		end

	anchor (pt: POINT): BUBBLE_SHADOW is
			-- Start dragging.
		require
			ok: pt /= Void
		deferred
		ensure
			may_be_void: -- outside implies Result = Void
		end

	resize (ns: BUBBLE_SHADOW) is
			-- Change object size.
		require
			ok: ns /= Void
		deferred
		end

feature -- Draw

	draw (gd: GRAPHIC_DEVICE) is
			-- Draw object.
		require
			ok: gd /= Void and then gd.is_ready
		deferred
		end

	draw_outline (gd: GRAPHIC_DEVICE; np: BUBBLE_SHADOW) is
			-- Draw object outline (XOR) for object moves.
		require
			gd_ok: gd /= Void and then gd.is_ready
			ol_size_ok: np /= Void
		deferred
		end

end -- class BUBBLE_OBJECT

