-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Bubble
	description: "Frame object (box enclosing several classes)"
	interface: client

class FRAME 

inherit 
	BOX
		end

creation
	make

feature {NONE}
	
	make is
			-- Creation.
		do
			!!position.make
			position.set_width (Default_width)
			position.set_height (Default_height)
		end

feature {NONE}
	
	Default_width: INTEGER is 180
	Default_height: INTEGER is 120
	Corner_height: INTEGER is 25
	Inside_margin: INTEGER is 3

feature -- Properties
	
	properties (ow: BUBBLE_DRAW): MODELESS_DIALOG is
			-- No properties for frame (should have title?)
		do
		end

feature
	
	copy (other: like Current) is
		do
			!!position.make
			position.copy (other.position)

			handles := Void
		end

	is_equal (other: like Current): BOOLEAN is
		do
			Result := position.is_equal (other.position) 
		end

feature -- ID

	type: STRING is
		do
			Result := "frame"
		end

feature {NONE} -- Serialisation

	string_to_object (tm: P_TEXT_OBJECT_MANAGER) is
		local
			version: INTEGER
		do
			make

			tm.get_integer
			version := tm.last_integer
			
			tm.get_object (position)

			if version /= 1 or not tm.in_success then
				make
			end
		end

	object_to_string (tm: P_TEXT_OBJECT_MANAGER) is
		do
			tm.put_integer (1)
			tm.put_object (position)
		end

feature {NONE}
	

	real_position: RECTANGLE is
		require
			has_pos: position /= Void
		local
			low, up: POINT
		do
			Result := position

			if position.width > 3*Inside_margin and position.height > 3*Inside_margin then
				
				low := position.lower
				low.set_x (low.x+Inside_margin)
				low.set_y (low.y+Inside_margin)
				up := position.upper
				up.set_x (up.x-Inside_margin)
				up.set_y (up.y-Inside_margin)

				!!Result.make
				Result.set_points (low, up)
			end
		end

feature -- Drawing

	draw (dev: GRAPHIC_DEVICE) is
		local
			dr: DRAWABLE_ROUND_RECTANGLE
			fore_clr: RGB_COLOR
			gr: GRAPHICS_ROUTINES
			pen: GEOMETRIC_PEN
		do
			!!gr

			!!fore_clr.black

			!!pen.make
			pen.set_color (fore_clr)
			pen.set_line_dot
			dev.set_pen (pen)
			dev.set_brush (gr.void_brush)
			
			!!dr.make
			dr.set_position (real_position)
			dr.set_corner (Corner_height, Corner_height)
			dev.draw (dr)
		end

	draw_outline (dev: GRAPHIC_DEVICE; any_shadow: BUBBLE_SHADOW) is
		local
			dr: DRAWABLE_ROUND_RECTANGLE
			box: BOX_SHADOW
			mode: GRAPHIC_MODE
			gr: GRAPHICS_ROUTINES
			outline_clr: RGB_COLOR
		do
			!!gr

			!!outline_clr.normal_blue

			box ?= any_shadow
			check good_box: box /= Void end

			!!mode.make

			mode.inverse
			dev.set_mode (mode)

			dev.set_pen (gr.color_pen (outline_clr))
			dev.set_brush (gr.void_brush)

			!!dr.make
			dr.set_position (box.position)
			dr.set_corner (Corner_height, Corner_height)
			dev.draw (dr)

			mode.pen
			dev.set_mode (mode)
		end

end
