-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Samples
	description: "Clock"
	interface: client

class CLOCK

inherit
	APPLICATION
		redefine
			when_created, when_drawn, when_resized
		end
	GUI_COMMAND
		rename
			execute as when_timer_event
		end

creation
	make

feature {NONE}
	
	time: P_TIME

	pen_color, back_color: COLOR
	hour_width, minute_width: INTEGER

	Hour_size: DOUBLE is 0.6
	Canonical_hour_width: INTEGER is 6

	Minute_size: DOUBLE is 0.8
	Canonical_minute_width: INTEGER is 4

	Frame_size: DOUBLE is 0.9
	
feature {NONE} -- Time
	
	get_time is
			-- Time has changed, get from system.
		local
			cl: F_CLOCK
		do
			!!cl
			cl.get_local_time
			time := cl.last_time.time
		end

feature {NONE} -- Drawing
	
	arm_spot (frame: RECTANGLE; angle: INTEGER; sz: DOUBLE): POINT is
		require
			good_point: frame /= Void
			good_angle: angle >= 0 and angle < 60 -- Pi/30 is the unit
			good_size: sz >= 0.0 and sz <= 1.0
		local
			math: P_MATH
			sys: P_SYSTEM
			actual_size: INTEGER
			actual_angle: DOUBLE
			ct: POINT
		do
			!!math
			!!sys

			actual_size := sys.floor (sz * ( (frame.width).min (frame.height) // 2))
			actual_angle := (math.Pi_value/30.0) * angle

			ct := frame.center

			!!Result.make
			Result.set_x (ct.x + sys.floor (math.sine (actual_angle) * actual_size))
			Result.set_y (ct.y - sys.floor (math.cosine (actual_angle) * actual_size))
		end

	draw_clock is
			-- Draw clock frame.
		require
			device_ok: device.is_ready
		local
			spot: DRAWABLE_ELLIPSE
			frame, rect_hour, rect_minute: RECTANGLE
			pt: POINT
			i: INTEGER
			gr: GRAPHICS_ROUTINES
		do
			-- prepare device
			!!gr
			gr.set_pen_and_brush (device, pen_color)

			-- spot bounding rectangle
			!!rect_hour.make
			rect_hour.set_width (hour_width)
			rect_hour.set_height (hour_width)
			!!rect_minute.make
			rect_minute.set_width (minute_width)
			rect_minute.set_height (minute_width)

			-- centre
			frame := client_size

			-- draw spots
			from
				!!spot.make
				i := 0
			variant 60 - i until i > 59 loop
				
				pt := arm_spot (frame, i, Frame_size)
				if (i \\ 5) = 0 then -- hour
					rect_hour.translate_center (pt)
					spot.set_bounding_rectangle (rect_hour)
				else -- minute
					rect_minute.translate_center (pt)
					spot.set_bounding_rectangle (rect_minute)
				end
			
				device.draw (spot)

				-- next
				i := i + 1
			end
		end

	draw_hour is
		require
			device_ok: device.is_ready
		local
			cp: GEOMETRIC_PEN
			dl: DRAWABLE_LINE
			cs: RECTANGLE
		do
			!!cp.make
			cp.set_color (pen_color)
			cp.set_width (hour_width)

			cs := client_size

			!!dl.make
			dl.set_start (cs.center)
			dl.set_final (arm_spot (cs, (time.hour \\ 12) * 5 + time.minute // 12, Hour_size))
			
			device.set_pen (cp)
			device.draw (dl)
		end

	draw_minute is
		require
			device_ok: device.is_ready
		local
			cp: GEOMETRIC_PEN
			dl: DRAWABLE_LINE
			cs: RECTANGLE
		do
			!!cp.make
			cp.set_color (pen_color)
			cp.set_width (minute_width)

			cs := client_size

			!!dl.make
			dl.set_start (cs.center)
			dl.set_final (arm_spot (cs, time.minute, Minute_size))

			device.set_pen (cp)
			device.draw (dl)
		end

feature
	
	when_timer_event is
			-- Periodic event to update clock.
		local
			old_time: P_TIME
		do
			old_time := time
			get_time
			if time.hour /= old_time.hour or time.minute /= old_time.minute then
				paint
			end
		end

	when_created is
			-- Init.
		local
			timer: F_TIMER
			gr: GRAPHICS_ROUTINES
			icon: ICON
		do
			-- title and icon
			set_text ("Fenestra Clock")
			!!icon.make_resource ("ClockIcon")
			set_icon (icon)

			-- init colour
			!!gr
			!RGB_COLOR!pen_color.normal_red
			!RGB_COLOR!back_color.black
			set_background_color (gr.color_brush (back_color))

			-- init time
			get_time

			-- start timer
			!!timer.make
			timer.set_delay (1000)
			timer.set_command (Current)
			timer.start -- never stopped...
		end

	when_drawn is
			-- Draw clock.
		do
			draw_clock
			draw_hour
			draw_minute
		end

	when_resized is
			-- Window has been resized.
		local
			win_size: INTEGER
		do
			win_size := (client_size.width + client_size.height) // 2
			hour_width := (Canonical_hour_width * (win_size)) // 150
			minute_width := (Canonical_minute_width * (win_size)) // 150
			paint
		end

end -- class CLOCK
