-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Samples
	description: "Fenestra sample program: Hello World"
	interface: client

class APP

inherit
	APPLICATION
		redefine
			when_created,
			when_menu_help_changed,
			when_menu_help_reset
		end

creation
	make

feature {NONE}
	
	graphics, textobj, twinstar: DEMO_CHILD
	dialog_box: HELLODIALOG
	status: STATUS_BAR
	
	Status_text: STRING is "In girum imus nocte et consumimur igni"

feature {PUBLIC_NONE}
	
	when_created is
			-- Setup.
		local
			ico: ICON
			stack: VERTICAL_STACK_WINDOW
			split, vsplit: SPLIT_WINDOW
			toolbar: TOOLBAR
			sc: MOUSE_CURSOR
			button: BITMAP_BUTTON
			cmd: GUI_COMMAND
		do
			-- title
			set_text ("Fenestra Hello!")

			-- icon
			!!ico.make_resource ("HelloIcon")
			set_icon (ico)

			-- holder for toolbar/split/status
			!! stack.make (Current)
			
				-- create child windows
				!!toolbar.make (stack)
					!!button.make (toolbar)
					button.set_resource ("DialogBMP")
					!CMD_HELLO_DIALOG!cmd.make (Current)
					button.set_command (cmd)
				toolbar.add_element (button)
					!!button.make (toolbar)
					button.set_resource ("ExitBMP")
					!CMD_HELLO_QUIT!cmd.make (Current)
					button.set_command (cmd)
				toolbar.add_element (button)

			stack.set_first (toolbar)
			
				!!status.make (stack)
				status.set_status (Status_text)

			stack.set_last (status)

				!!vsplit.make_vertical (stack)
					!!split.make_horizontal (vsplit)
					vsplit.set_first (split)

						!GRAPHICS!graphics.make (split)
						split.set_first (graphics)

						!TEXT!textobj.make (split)
						split.set_second (textobj)

					!TWINSTAR!twinstar.make (vsplit)
					vsplit.set_second (twinstar)

			stack.set_variable (vsplit)
			stack.size_follows_parent

			-- menu
			install_menu
		end

	when_menu_help_changed (s: STRING) is
			-- Change status bar when menu help changed.
		do
			status.set_status (s)
		end

	when_menu_help_reset is
			-- Reset contextual menu.
		do
			status.set_status (Status_text)
		end

feature -- Commands

	open is
			-- Open file dialog.
		local
			openbox: FILE_OPEN_DIALOG
		do
			!!openbox.make (Current)
			openbox.add_filter ("All files", "*.*")
			openbox.add_filter ("Text files", "*.doc;*.txt")
			openbox.execute
			
		end

	save is
			-- Close file dialog.
		local
			savebox: FILE_SAVE_DIALOG
		do
			!!savebox.make (Current)
			savebox.add_filter ("Text files", "*.txt")
			savebox.execute
		end

	dialog is
			-- Sample dialog box.
		do
			if dialog_box /= Void and then dialog_box.is_valid then
				dialog_box.activate
			else
				!!dialog_box.make (Current)
			end
		end

	quit is
			-- Quit.
		do
			close
		end

feature {NONE}
	
	install_menu is
			-- Setup menu.
		local
			menubar: MENU_BAR
			popup: POPUP_MENU
			item: MENU_ITEM
			hlpcommand: HELP_COMMAND
			cmd: GUI_COMMAND
		do
			!!menubar.make (Current)
			
			!!popup.make (Current)
			popup.set_name ("&File")

				!!item.make ("&Open...")
				item.set_help ("Standard file open dialog.")
				!CMD_HELLO_OPEN!cmd.make (Current)
				item.set_command (cmd)
				popup.add_item (item)
				
				!!item.make ("S&ave...")
				item.set_help ("Standard file save dialog.")
				!CMD_HELLO_SAVE!cmd.make (Current)
				item.set_command (cmd)
				popup.add_item (item)

				popup.add_separator
				
				!!item.make ("&Dialog box")
				item.set_help ("Example dialog box.")
				!CMD_HELLO_DIALOG!cmd.make (Current)
				item.set_command (cmd)
				popup.add_item (item)
				
				popup.add_separator

				!!item.make ("Q&uit")
				item.set_help ("Exit program.")
				!CMD_HELLO_QUIT!cmd.make (Current)
				item.set_command (cmd)
				popup.add_item (item)
			
			menubar.add_popup (popup)

			menubar.add_popup (graphics.popup)
			menubar.add_popup (textobj.popup)
			menubar.add_popup (twinstar.popup)

			!!popup.make (Current)
			popup.set_name ("&Help")

				!!item.make ("H&elp on Help")
				!HELP_ON_HELP!hlpcommand.make (Current)
				hlpcommand.set_helpfile ("winhelp.hlp")
				item.set_command (hlpcommand)
				popup.add_item (item)

			menubar.add_popup (popup)

			menubar.attach
		end

end -- class APP

