-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

class GRAPHICS

inherit
	DEMO_CHILD
		redefine
			when_created, when_closed, when_drawn
		end
	GUI_COMMAND
		rename
			execute as heart_beat
		end

creation
	make, make_child

feature
	
	make_popup (par: OVERLAPPED_WINDOW) is
		local
			hitem: HEAD_MENU_ITEM
			item: MENU_ITEM
			cmd: GRAPHICS_COMMAND
		do
			!!popup.make (par)
			popup.set_name ("&Graphics")

			!!hitem.make ("&Stars")
			!GRAPHICS_STAR!cmd.make (Current)
			hitem.set_command (cmd)
			popup.add_item (hitem)

			set_figure (cmd) -- default
			
			!!item.make ("&Circle")
			!GRAPHICS_CIRCLE!cmd.make (Current)
			item.set_command (cmd)
			item.set_leader (hitem)
			popup.add_item (item)

			!!item.make ("&Rectangle")
			!GRAPHICS_RECTANGLE!cmd.make (Current)
			item.set_command (cmd)
			item.set_leader (hitem)
			popup.add_item (item)
			-- more:
		end

	set_figure (fig: GRAPHICS_COMMAND) is
		require
			ok: fig /= Void
		do
			figure := fig
		end

feature {NONE} 
	
	Maximum_size: INTEGER is 400 -- 4 cm

feature {NONE} -- Attributes

	timer: F_TIMER
	times: INTEGER -- Number of objects displayed
	figure: GRAPHICS_COMMAND
	palsize: INTEGER -- palette size
	back_color: COLOR

	rand: P_RANDOM

feature
	
	heart_beat is
		local
			color: COLOR
			p: POINT
			gr: GRAPHICS_ROUTINES
		do
			check valid: is_valid end -- check timer leaks
			if not is_iconic and not size.is_thin then

				-- redraw from scratch occasionally
				if times > 100 then
					paint
					times := 0
				else
					times := times + 1
				end

				-- prepare color
				if palsize > 0 then
					-- scale of green
					rand.get_number (1, palsize)
					!PALETTE_COLOR!color.make_index (rand.last_number)
				else
					-- scale of blue
					rand.get_number (1, 255)
					!RGB_COLOR!color.rgb (0, 0, rand.last_number)
				end

				-- prepare pen/brush
				!!gr
				device.set_pen (gr.color_pen (color))
				device.set_brush (gr.color_brush (color))

				-- Note: it is generally not recommended to draw outside 
				-- when_drawn as redrawing will not be handled correctly 
				-- but that's acceptable for the sake of this dynamic example.
				
				device.prepare
				
				-- centre of figure
				!!p.make
				p.set_x (size.width)
				p.set_y (size.height) 
				
				-- convert to cartesian coordinates
				p := device.device_to_logical (p)

				rand.get_number (1, p.x)
				p.set_x (rand.last_number)
				rand.get_number (1,- p.y)
				p.set_y (- rand.last_number)
				
				rand.get_number (Maximum_size // 4, Maximum_size)
				device.draw (figure.figure (p, rand.last_number))
				
				device.release
			end
		end

feature
	
	when_created is
		local
			pm: PAGE_MAPPING
			pal: LOGICAL_PALETTE
			i, n: INTEGER
			col: RGB_COLOR
			clock: F_CLOCK
		do
			-- init random number
			!!clock clock.get_tick
			!!rand.make (clock.last_tick)
			
			-- mapping (centimetres)
			!!pm.make
			pm.low_metric
			device.set_mapping (pm)
			
			-- border
			set_border (false)

			-- initialise timer (calls heart_beat)
			!!timer.make
			timer.set_delay (500) -- .5 seconds
			timer.set_command (Current)
			timer.start

			-- create palette
			device.prepare

			if device.capabilities.has_palette then
				palsize := (device.capabilities.palette_size).max (255)
				
				!!pal.make (palsize)
				from
					i := 1
					n := 1
				variant
					palsize + 1 - n
				until 
					n >= palsize -- so that n + 1 <= palsize inside
				loop 

					!!col.rgb (0, 256-i, 0) -- dark greens
					pal.set_color (n, col)
					n := n + 1
					
					!!col.rgb (i, 255, i) -- light greens
					pal.set_color (n, col)
					n := n + 1
					
					i := i + 1
				end
				device.set_palette (pal)
			else
				palsize := 0
			end
			
			device.release
			
			-- background colour
			!RGB_COLOR!back_color.black
			
			times := 0
		end

	when_closed is
		do
			timer.stop
		end

	when_drawn is
		local
			dr: DRAWABLE_RECTANGLE
			r: RECTANGLE
			gr: GRAPHICS_ROUTINES
		do
			!!gr
			device.set_pen (gr.color_pen (back_color))
			device.set_brush (gr.color_brush (back_color))
			
			!!r.make_points (device.device_to_logical (client_size.lower),
				device.device_to_logical (client_size.upper))
			
			!!dr.make
			dr.set_position (r)
			device.draw (dr)
		end

end -- class GRAPHICS
