-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

class TEXT

inherit
	DEMO_CHILD
		redefine
			when_created, when_drawn, when_resized, 
			when_character_entered, when_key_pressed,
			when_mouse_clicked
		end

creation
	make, make_child

feature
	
	make_popup (ow: OVERLAPPED_WINDOW) is
		local
			item: MENU_ITEM
			hitem: HEAD_MENU_ITEM
			cmd: GUI_COMMAND
		do
			!!popup.make (ow)
			popup.set_name ("&Text")

			!CMD_TEXT!cmd.make (Current, "Hello!")
			cmd.execute -- default
			!!hitem.make ("E&nglish")
			hitem.set_command (cmd)
			popup.add_item (hitem)

			!CMD_TEXT!cmd.make (Current, "Guten Tag!")
			!!item.make ("&German")
			item.set_command (cmd)
			item.set_leader (hitem)
			popup.add_item (item)

			!CMD_TEXT!cmd.make (Current, "Bonjour!")
			!!item.make ("&French")
			item.set_command (cmd)
			item.set_leader (hitem)
			popup.add_item (item)

			!CMD_TEXT!cmd.make (Current, "Saluton!")
			!!item.make ("&Esperanto")
			item.set_command (cmd)
			item.set_leader (hitem)
			popup.add_item (item)

			!CMD_TEXT_OWN!cmd.make (Current)
			!!item.make ("&Own text")
			item.set_command (cmd)
			item.set_leader (hitem)
			popup.add_item (item)
		
			popup.add_separator
			
			!CMD_TEXT_FONT!cmd.make (Current)
			!!item.make ("Fon&t")
			item.set_command (cmd)
			popup.add_item (item)

			!CMD_TEXT_COLOR!cmd.make (Current)
			!!item.make ("Background &colour")
			item.set_command (cmd)
			popup.add_item (item)
		end

feature {NONE} -- Attributes

	hellotext: STRING
	point: POINT
	owntext: BOOLEAN
	font: LOGICAL_FONT
	color: COLOR

feature -- Menu choices
	
	set_hellotext (newtext: STRING) is
		require
			ok: newtext /= Void
		do
			owntext := false
			hellotext := newtext
			paint
		ensure
			keep_reference: newtext = hellotext
		end

	set_own is
		do
			owntext := true
			!!hellotext.make (0)
			focus
			paint
		end

	change_font is
		local
			fd: FONT_DIALOG
		do
			!!fd.make (overlapped_parent)
			if font /= Void then
				fd.set_font (font)
			end
			fd.execute
			if fd.has_selection then
				font := fd.font
				device.set_font (font)
				paint
			end
		end

	change_color is
		local
			cd: COLOR_DIALOG
		do
			!!cd.make (overlapped_parent)
			if color /= Void then
				cd.set_color (color)
			end
			cd.execute
			if cd.has_selection then
				color := cd.color
				device.set_background_color (color)
				paint
			end
		end

feature {PUBLIC_NONE}
	
	when_created is
		do
			!!hellotext.make (0)
			!!point.make_coord (50, 50)
		end

	when_drawn is
			-- Draw
		local
			--dr: DRAWABLE_RECTANGLE
			dp: DRAWABLE_POINT
			dt: DRAWABLE_TEXT
			cs: RECTANGLE
			red: RGB_COLOR
		do
			cs := client_size
			--!!dr.make
			--dr.set_position (client_size)
			--device.draw (dr)

			!!dt.make
			dt.set_position (point)
			dt.set_text (hellotext)
			dt.set_area (cs)
			device.draw (dt)

			!!red.normal_red
			!!dp.make
			dp.set_position (cs.upper)
			dp.set_color (red)
			device.draw (dp)
		end

	when_resized is
		do
			paint -- clear screen
		end

	when_character_entered (c: CHARACTER) is
		do
			if owntext then
				hellotext.append_character (c)
				paint
			end
		end

	when_key_pressed (key: VIRTUAL_KEY) is
		do
			if key.is_return and then owntext then
				hellotext.copy ("")
				paint 
			end
		end

	when_mouse_clicked (p: POINT) is
		do
			device.prepare
			point.copy (device.device_to_logical (p))
			device.release
			paint
		end
	
invariant
	valid_data: is_created implies (point /= Void and hellotext /= Void)

end -- class TEXT
