-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

class TWINSTAR

inherit
	DEMO_CHILD
		redefine
			when_drawn, when_mouse_clicked, when_resized
		end

creation
	make

feature
	
	make_popup (ow: OVERLAPPED_WINDOW) is
			-- Create associated popup menu.
		local
			item: MENU_ITEM
			cmd: GUI_COMMAND
		do
			!!popup.make (ow)
			popup.set_name ("T&wins")

			!CMD_PASTE!cmd.make (Current)
			!!item.make ("&Paste metafile")
			item.set_command (cmd)
			popup.add_item (item)
		end
	
feature {PUBLIC_NONE}
	
	when_drawn is
			-- Draw client area.
		do
			draw_stars (device)
		end

	when_mouse_clicked (p: POINT) is
			-- Paste.
		do
			paste
		end
	
	when_resized is
			-- Window has been resized.
		do
			paint -- clear screen
		end

feature 
	
	paste is
			-- Paste to metafile clipboard.
		local
			clip: CLIPBOARD_METAFILE
			metafile: METAFILE_DEVICE
			mapmode: PAGE_MAPPING
			pt: POINT
		do
			device.prepare
			
			-- convert window client size to metafile size
			!!mapmode.make
			!!pt.make_coord (client_size.width, - client_size.height)
			
			mapmode.high_metric
			device.set_mapping (mapmode)
			
			pt := device.device_to_logical (pt)

			mapmode.text
			device.set_mapping (mapmode)
			
			-- create metafile
			!!metafile.make_memory (device, pt.x, pt.y)
			device.release

			-- draw in metafile
			draw_stars (metafile)

			-- paste
			!!clip.make (Current)
			clip.set_metafile (metafile)
			clip.put

			check clip_done: clip.has_data end
		end

feature {NONE}
	
	draw_stars (output: GRAPHIC_DEVICE) is
			-- Actual device independent drawing.
		local
			cpt: POINT
			back: DRAWABLE_RECTANGLE
			star: FIVE_BRANCH_STAR
			poly: DRAWABLE_POLYGON
			h, w: INTEGER
			gr: GRAPHICS_ROUTINES
			clr: RGB_COLOR
		do
			!!gr

			-- Draw background
			!!clr.white
			output.set_brush (gr.color_brush (clr))
			!!back.make
			back.set_position (client_size)
			output.draw (back)

			-- Draw stars
			!!clr.dark_grey
			output.set_brush (gr.color_brush (clr))

			h := size.height
			w := size.width
			
			!!cpt.make
			!CROSS_STAR!star.make (cpt, (2*h//6).min (w//6)) -- keep ref on `cpt'
			
			cpt.set_xy (w//4, h//2)
			poly := star.polygon
			poly.set_filling_mode_winding
			output.draw (poly)

			cpt.set_x (3*w//4)
			poly := star.polygon
			poly.set_filling_mode_alternate
			output.draw (poly)
		end

end -- class TWINSTAR
