-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Tutorial, Tictactoe
	description: "Tic-tac-toe model"
	interface: client

class TICTACTOE

creation
	make

feature {NONE} -- Data

	board: ARRAY[ARRAY[INTEGER]]
			-- The board representation.

	level: INTEGER

	winner: INTEGER

	Empty, Computer, User, Both: INTEGER is unique
			-- Piece constants

feature -- Constant(s)
	
	Max_level: INTEGER is 4;
			-- Maximum computer intelligence.

feature {NONE} -- Creation

	make is
		do
			level := 1
			reset
		end

feature -- Reset

	reset is
			-- Create/initialise.
		local
			col, row: INTEGER
			row_array: ARRAY[INTEGER]
		do
			winner := Empty

			-- make board
			from
				col := 1
				!!board.make (1, 3)
			variant
				4 - col
			until
				col > 3
			loop
				-- make a row
				from
					row := 1
					!!row_array.make (1, 3)
				variant
					4 - row
				until
					row > 3
				loop
					row_array.put (Empty, row)
					row := row + 1
				end
				board.put (row_array, col)

				-- forth
				col := col + 1
			end
		end

feature -- Option(s)

	set_level (lvl: INTEGER) is
			-- Level (1: Easy, Max_level: Difficult).
		require
			valid: lvl > 0 and lvl <= Max_level
		do
			level := lvl
		ensure
			done: level = lvl
		end

feature {NONE} -- Board access

	square (col, row: INTEGER): INTEGER is
			-- Retrieve square.
		require
			ok: is_square (col, row)
		do
			Result := board.item (col).item (row)
		ensure
			done: Result = Empty or Result = Computer or Result = User
		end


	set_square (col, row: INTEGER; new: INTEGER) is
			-- Set square.
		require
			valid_coord: is_square (col, row)
			valid_new: new = Empty or new = Computer or new = User
		do
			board.item (col).put (new, row)
		end
		
feature -- Status
	
	is_square (col, row: INTEGER): BOOLEAN is
			-- Is this (col, row) correct?
		do
			Result := col >= 1 and col <= 3 and row >= 1 and row <= 3
		ensure
			done: Result = (col >= 1 and col <= 3 and row >= 1 and row <= 3)
		end

	is_empty (col, row: INTEGER): BOOLEAN is
			-- Is this square empty?
		require
			in: is_square (col, row)
		do
			Result := square (col, row) = Empty
		end

	is_user (col, row: INTEGER): BOOLEAN is
			-- Is this a user's square?
		require
			in: is_square (col, row)
		do
			Result := square (col, row) = User
		end

	is_computer (col, row: INTEGER): BOOLEAN is
			-- Is this a computer's square?
		require
			in: is_square (col, row)
		do
			Result := square (col, row) = Computer
		end

	is_finished: BOOLEAN is
			-- Finished?
		do
			Result := winner /= Empty
		end

	is_user_winner: BOOLEAN is
			-- Who wins?
		require
			ok: is_finished
		do
			Result := winner = User
		end

	is_computer_winner: BOOLEAN is
		require
			ok: is_finished
		do
			Result := winner = Computer
		end

feature -- Action
	
	move (col, row: INTEGER) is
			-- User plays on (col, row).
		require
			notend: not is_finished
			ok: is_square (col, row)
			empty: is_empty (col, row)
		do
			set_square (col, row, User)
			check_winner
		end

	computer_move is
			-- Computer plays.
		require
			notend: not is_finished
		local
			col, row: INTEGER
			done: BOOLEAN
		do

			-- top trick
			if level > 3 and then is_empty (2, 2) then
				internal_computer_move (2, 2)
				done := true
			end

			-- check rows/columns
			if level > 1 then
				from
					row := 1
				variant
					4 - row
				until
					done or else row > 3
				loop
					done := check_row (row)
					if not done then
						done := check_column (row)
					end
					row := row + 1
				end
			end

			-- diagonals
			if level > 2 and then not done then
				done := check_diagonals
			end
		
			-- default (first empty box)
			if not done then
				from col := 1 variant 4 - col
				until done or else col > 3 loop
					from row := 1 variant 4 - row
					until done or else row > 3 loop
						if is_empty (col, row) then
							internal_computer_move (col, row)
							done := true
						end
						row := row + 1
					end
					col := col + 1
				end
			end
			
			-- not possible
			check done: done end
		end

feature {NONE} -- Game logic
		
	check_winner is
			-- Check if there is a winner.
		require
			not is_finished
		local
			i, j: INTEGER
			emptybox: BOOLEAN
		do
			-- find winner
			-- col/row
			from i := 1 variant 4 - i until
				winner /= Empty or else i > 3
			loop
				from 
					j := 1
				variant 4 - j until emptybox or j > 3 loop
					emptybox := is_empty (i, j)
					j := j + 1
				end

				if not is_empty (i, 1) and then threeway_equal (square (i, 1), square (i, 2), square (i, 3)) then
					winner := square (i, 1)
				end
				if winner = Empty and then not is_empty (1, i) and then threeway_equal (square (1, i), square (2, i), square (3, i)) then
					winner := square (1, i)
				end
				i := i + 1
			end

			-- diagonals
			if winner = Empty and then not is_empty (2, 2) and then 
					 (threeway_equal (square (1, 1), square (2, 2), square (3, 3)) 
				or
					threeway_equal (square (1, 3), square (2, 2), square (3, 1))) 
			then
				winner := square (2, 2)
			end
			
			-- end with no winner
			if not emptybox and winner = Empty then
				winner := Both
			end
		end

	threeway_equal (a, b, c: INTEGER): BOOLEAN is do Result := (a = b) and (a = c) end
	
	check_row (row: INTEGER): BOOLEAN is
			-- Check if row has 2 user or computer elements and fill the third if so.
		local
			col: INTEGER
			good, bad, free: INTEGER
		do
			from col := 1 variant 4 - col until
				col > 3 
			loop
				if is_user (col, row) then
					bad := bad + 1
				elseif is_computer (col, row) then
					good := good + 1
				elseif is_empty (col, row) then
					free := col
				end
				col := col + 1
			end
			if (good = 2 or else bad = 2) and then free > 0 then
				Result := true
				internal_computer_move (free, row)
			end
		end

	check_column (col: INTEGER): BOOLEAN is
			-- Check if column has 2 user or computer elements and fill the third if so.
		local
			row: INTEGER
			good, bad, free: INTEGER
		do
			from row := 1 variant 4 - row until
				row > 3
			loop
				if is_user (col, row) then
					bad := bad + 1
				elseif is_computer (col, row) then
					good := good + 1
				elseif is_empty (col, row) then
					free := row
				end
				row := row + 1
			end
			if (good = 2 or else bad = 2) and then free > 0 then
				Result := true
				internal_computer_move (col, free)
			end
		end

	check_diagonals: BOOLEAN is
			-- Play on diagonal lines.
		local
			i: INTEGER
			good, bad, free: INTEGER
			good2, bad2, free2: INTEGER
		do
			from i := 1 variant 4 - i until
				i > 3
			loop
				if is_user (i, i) then
					bad := bad + 1
				elseif is_computer (i, i) then
					good := good + 1
				elseif is_empty (i, i) then
					free := i
				end

				if is_user (4-i, i) then
					bad2 := bad2 + 1
				elseif is_computer (4-i, i) then
					good2 := good2 + 1
				elseif is_empty (4-i, i) then
					free2 := i
				end
				i := i + 1
			end

			if (good = 2 or else bad = 2) and then free > 0 then
				Result := true
				internal_computer_move (free, free)
			elseif (good2 = 2 or else bad2 = 2) and then free2 > 0 then
				Result := true
				internal_computer_move (4 - free2, free2)
			end
		end

	internal_computer_move (col, row: INTEGER) is
		require
			notend: not is_finished
			ok: is_square (col, row)
			empty: is_empty (col, row)
		do
			set_square (col, row, Computer)
			check_winner
		end

invariant
	board: board /= Void

end -- class TICTACTOE
