-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Tutorial, Tictactoe
	description: "Tic-tac-toe main window"
	interface: client

class VIEW

inherit
	APPLICATION
		redefine
			when_created, 
			when_mouse_clicked,
			when_drawn,
			when_resized
		end

creation
	make

feature {NONE} -- Private attribute(s)

	model: TICTACTOE
	board_size: INTEGER
	painting: BOOLEAN

feature {PUBLIC_NONE} -- Events

	when_created is
			-- Window initialised.
		local
			ico: ICON
		do
			-- Initialise model.
			!!model.make
			set_level (1)

			-- Set title
			set_text ("Tic-tac-toe")
			
			-- Set icon
			!!ico.make_resource ("AppIcon")
			set_icon (ico)

			-- Menu bar.
			create_menu_bar
		end

	when_mouse_clicked (pt: POINT) is
			-- Mouse has been clicked.
		local
			col, row, cell: INTEGER
			sys: F_SYSTEM
		do
			if model.is_finished then
				-- Last game is finished, start a new one.
				new_game
			else
				-- Retrieve square.
				cell := board_size // 3
				col := (pt.x - cell) // cell + 1
				row := (pt.y - cell // 2) // cell + 1
				
				if model.is_square (col, row) and then model.is_empty (col, row) then
					-- User clicked on a free square.
					model.move (col, row)
					if not model.is_finished then
						model.computer_move
					end
					paint
				else
					-- Bad place.
					!!sys
					sys.beep
				end
			end
		end

	when_drawn is
			-- Window drawn.
		local
			win_size: INTEGER
			r: RECTANGLE
		do
			-- Initialise window size.
			win_size := (client_size.width).min (client_size.height)
			board_size := win_size * 3 // 5
			
			!!r.make
			r.set_width (win_size)
			r.set_height (win_size)

			-- Resize. (`painting' avoids recursion)
			painting := true
			set_client_size (r)
			painting := false

			-- draw grid
			draw_board

			-- draw square
			draw_squares

			-- draw winner
			if model.is_finished then
				draw_winner
			end
		end

	when_resized is
			-- Window has been received.
		do
			if not painting then
				paint
			end
		end

feature -- Public attribute(s)

	firstmove_computer: BOOLEAN
			-- Is the computer going to move first?

	level: INTEGER
			-- Current level.

feature -- Options

	set_firstmove_computer is
			-- Computer moves first.
		do
			firstmove_computer := true
		end

	set_firstmove_user is
			-- User moves first.
		do
			firstmove_computer := false
		end
	
	set_level (lvl: INTEGER) is
			-- Set computer intelligence. 
		do
			level := lvl
			model.set_level (lvl)
		end

feature {NONE} -- Menu

	create_menu_bar is
			-- Initialise the menu bar.
		local
			bar: MENU_BAR
			popup: POPUP_MENU
			item: MENU_ITEM
			command: GUI_COMMAND
		do
			!!bar.make (Current)

			!!popup.make (Current)
			popup.set_name ("&Game")
			
			!CMD_NEW!command.make (Current)
			!!item.make ("&New")
			item.set_command (command)
			popup.add_item (item)

			!CMD_OPTIONS!command.make (Current)
			!!item.make ("&Options...")
			item.set_command (command)
			popup.add_item (item)
			
			popup.add_separator

			!CMD_QUIT!command.make (Current)
			!!item.make ("&Quit")
			item.set_command (command)
			popup.add_item (item)
			
			bar.add_popup (popup)
			bar.attach
		end

feature {NONE} -- Dialog
	
	dialog: GAME_DIALOG

feature {GUI_COMMAND} -- Menu item(s)

	new_game is
			-- Start new game.
		do
			model.reset
			if firstmove_computer then
				model.computer_move
			end
			paint
		end

	quit is
			-- Exit application.
		do
			close
		end

	options is
			-- Set options.
		do
			-- Modeless dialog, menu may be used when dialog open.
			if dialog /= Void and then dialog.is_valid then
				dialog.activate
			else
				!!dialog.make (Current)
			end
		end

feature {NONE} -- Drawing

	draw_board is
			-- Draw row.
		local
			clr: RGB_COLOR
			spen: SIMPLE_PEN
			cell, x, y, i: INTEGER
			dln: DRAWABLE_LINE
			pt: POINT
		do
			!!pt.make
			!!dln.make

			cell := board_size // 3
			
			-- set color/pen
			!!clr.black
			
			!!spen.make
			spen.set_color (clr)
			spen.set_width (2)
			
			device.set_pen (spen)
			
			-- draw board
			from i := 0 variant 4 - i until i > 3 loop
				-- draw row
				y := (cell // 2) + (i*cell)

				pt.set_xy (cell, y)
				dln.set_start (pt)
				
				pt.set_xy (4*cell, y)
				dln.set_final (pt)

				device.draw (dln)
				
				-- draw colum
				x := (i+1) * cell

				pt.set_xy (x, cell // 2)
				dln.set_start (pt)
				
				pt.set_xy (x, cell // 2 + 3 * cell)
				dln.set_final (pt)

				device.draw (dln)

				-- forth
				i := i + 1
			end
		end

	draw_squares is
			-- Draw all squares.
		local
			i, j: INTEGER
		do
			from i := 1 variant 4 - i until i > 3 loop
				from j := 1 variant 4 - j until j > 3 loop
					if not model.is_empty (i, j) then
						draw_square (i, j)
					end
					j := j + 1
				end
				i := i + 1
			end
		end
	
	draw_square (col, row: INTEGER) is
			-- Draw one square.
		require
			ok: model.is_square (col, row)
		local
			pt: POINT
			rect: RECTANGLE
			cell, subcell: INTEGER
			dcircle: DRAWABLE_ELLIPSE
			dline: DRAWABLE_LINE
			clr: RGB_COLOR
			gpen: GEOMETRIC_PEN
			gr: GRAPHICS_ROUTINES
		do
			cell := board_size // 3
			subcell := cell * 7 // 10
			
			-- cell centre
			!!pt.make_coord (col*cell + cell//2, row*cell)
			
			!!rect.make
			rect.set_width (subcell)
			rect.set_height (subcell)
			rect.translate_center (pt)

			if model.is_user (col, row) then
				-- set color/brush/pen
				!!gr
				!!clr.normal_red
				device.set_brush (gr.color_brush (clr))
				device.set_pen (gr.color_pen (clr))
				
				-- draw circle
				!!dcircle.make
				dcircle.set_bounding_rectangle (rect)
				device.draw (dcircle)
			
			elseif model.is_computer (col, row) then
				-- set color/brush/pen
				!!clr.normal_blue
				!!gpen.make
				gpen.set_color (clr)
				gpen.set_end_round
				gpen.set_width (cell // 8)
				device.set_pen (gpen)

				-- draw cross
				!!dline.make
				dline.set_start (rect.lower)
				dline.set_final (rect.upper)
				device.draw (dline)

				pt.set_xy (rect.lower.x, rect.upper.y)
				dline.set_start (pt)
				pt.set_xy (rect.upper.x, rect.lower.y)
				dline.set_final (pt)
				device.draw (dline)
			end
		end

	draw_winner is
			-- Draw winner banner.
		require
			ok: model.is_finished
		local
			txt: STRING
			rect: RECTANGLE
			dtext: DRAWABLE_TEXT
			pt: POINT
			clr: RGB_COLOR
		do
			!!clr.black
			
			if model.is_user_winner then
				txt := "You win!"
			elseif model.is_computer_winner then
				txt := "Computer wins!"
			else
				txt := "Nobody wins."
			end
			
			!!dtext.make
			dtext.set_color (clr)
			dtext.set_text (txt)
			
			!!pt.make_coord (5 * board_size // 6, 3 * board_size // 2)
			rect := device.text_extent (dtext)
			rect.translate_center (pt)
			
			dtext.set_position (rect.lower)

			device.draw (dtext)
		end

end -- class VIEW
