-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, System
	description: "File properties"
	interface: client

class F_FILE_PROPERTIES

inherit
	WIN_API
		end
	WIN_FINDDATA_CONST
		end
	WIN_SYSTIME_CONST
		end

creation
	make

feature {CREATION_NONE} -- Creation
	
	make is
			-- Creation.
		do
			struct := Void
		end

feature {NONE}
	
	struct: F_MEMORY

feature {NONE}
	
	attrib (mask: INTEGER): BOOLEAN is
		do
			Result := struct.bit_and (struct.double_word (Fileattrib_offset), mask) /= 0
		end

	make_time (filetime: F_MEMORY): P_DATE_TIME is
		require
			valid: filetime /= Void
			size: filetime.size = 8
		local
			systime: F_MEMORY
		do
			!!systime.make (Systime_length)

			wrt.win_file_convtime (filetime.to_windows, systime.to_windows)

			!!Result.make
			Result.set (systime.word (Year_offset), systime.word (Month_offset),
						systime.word (Day_offset), systime.word (Hour_offset),
						systime.word (Minute_offset), systime.word (Second_offset))
			Result.set_timezone_bias (0)
		end

feature -- Status
	
	set_file (filename: STRING) is
			-- Set the file (or directory) name.
		require
			valid: filename /= Void
		do
			!!struct.make (Finddata_length)
			if wrt.win_file_prop (wrt.ew_string (filename).to_c, struct.to_windows) = false then
				struct := Void
			end
		ensure
			done: -- is_valid if successful
		end

	is_valid: BOOLEAN is
			-- Are the properties current?
		do
			Result := struct /= Void 
		end

feature -- Properties

	file_size: INTEGER is
			-- File size in bytes.
		require
			valid: is_valid
			notdir: not is_directory
			overflow: -- file_size_kb * 1024 < 2^Integer_bits
		do
			Result := struct.double_word (Sizelow_offset)
			
			--? proper processing of overflow?
			check 
				overflow_high: struct.double_word (Sizehigh_offset) = 0
				overflow_low: Result >= 0
			end
		ensure
			positive: Result >= 0
		end

	file_size_kb: INTEGER is
			-- File size in kilobytes.
		require
			valid: is_valid
			notdir: not is_directory
		do
			--? processing of larger files...
			Result := file_size // 1024
		end

	creation_time: P_DATE_TIME is
			-- Creation time of the file.
		require
			valid: is_valid
		do
			Result := make_time (struct.subset (Creation_offset, 8))
		end

	last_access_time: P_DATE_TIME is
			-- Last access time. This includes reading, writing, 
			-- and executing for programs.
		require
			valid: is_valid
		do
			Result := make_time (struct.subset (Lastaccess_offset, 8))
		end

	last_write_time: P_DATE_TIME is
			-- Last time the file was written.
		require
			valid: is_valid
		do
			Result := make_time (struct.subset (Lastwrite_offset, 8))
		end

	is_hidden: BOOLEAN is
			-- Is this file hidden?
		require
			valid: is_valid
		do
			Result := attrib (wrt.File_attribute_hidden)
		end
		
	is_read_only: BOOLEAN is
			-- Is this file read-only (attribute only, security permission 
			-- are not kept into account)?
		require
			valid: is_valid
		do
			Result := attrib (wrt.File_attribute_readonly)
		end
	
	is_system: BOOLEAN is
			-- Is this file used exclusively by the operating 
			-- system?
		require
			valid: is_valid
		do
			Result := attrib (wrt.File_attribute_system)
		end

	is_archive: BOOLEAN is
			-- Is this file marked as archived?
		require
			valid: is_valid
		do
			Result := attrib (wrt.File_attribute_archive)
		end
	
	is_directory: BOOLEAN is
			-- Is this object a directory?
		require
			valid: is_valid
		do
			Result := attrib (wrt.File_attribute_directory)
		end

end -- class F_FILE_PROPERTIES
