-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, System
	description: "File system services"
	interface: client

class F_FILE_SYSTEM

inherit
	WIN_API
		end

creation
	make

feature {CREATION_NONE} -- Creation

	make is
			-- Create object.
		do
			ldirectory := Void
			lcontent := Void
		end

feature -- File system

	free_disk_space (drivename: STRING): INTEGER is
			-- Return free disk space in KILOBYTES (1024 bytes).
		require
			valid_str: drivename /= Void
		do
			isize := wrt.win_file_dspace (wrt.ew_string (drivename).to_c)
			if isize < 0 then
				last_success := false
			else
				last_success := true
			end
		ensure
			done: -- last_success implies last_size set 
		end

feature -- Directories

	is_directory (pathname: STRING): BOOLEAN is
			-- Is `pathname' a file directory?
		require
			valid_str: pathname /= Void
		do
			Result := wrt.win_file_attrib (wrt.ew_string (pathname).to_c, wrt.File_attribute_directory)
		end

	create_directory (pathname: STRING) is
			-- Create directory `pathname'.
		require
			valid_str: pathname /= Void
			not_dir: not is_directory (pathname)
		do
			last_success := wrt.win_file_mkdir (wrt.ew_string (pathname).to_c)
		ensure
			done: last_success implies is_directory (pathname)
		end

	remove_directory (pathname: STRING) is
			-- Remove directory `pathname'.
		require
			valid_str: pathname /= Void
			not_dir: is_directory (pathname)
			empty: -- is_empty_directory (pathname)
		do
			last_success := wrt.win_file_rmdir (wrt.ew_string (pathname).to_c)
		ensure
			done: last_success implies not is_directory (pathname)
		end

	directory_content (pattern: STRING) is
			-- Get list of files and directories in this directory.
			-- May include * and ? wildcards.
		require
			valid_str: pattern /= Void
		local
			hdle: INTEGER
			lstr: STRING
			done: BOOLEAN
		do
			!P_LINKED_LIST[STRING]!lcontent.make
			
			wrt.win_file_ffirst (wrt.ew_string (pattern).to_c)
			lstr := wrt.win_laststring
			hdle := wrt.win_lastint1
			
			if hdle = wrt.Invalid_handle_value then
				last_success := false
			else
				lcontent.add (lstr)
				
				from
					done := false
				until
					done
				loop
					wrt.win_file_fnext (hdle)
					lstr := wrt.win_laststring
					done := not wrt.win_lastbool1 
					if not done then
						lcontent.add (lstr)
					end
				end

				wrt.win_file_fclose (hdle)
				last_success := true
			end
		ensure
			done: last_success implies last_content /= Void -- and set.
		end

	current_directory is
			-- Get current directory.
		do
			wrt.win_file_curdir
			ldirectory := wrt.win_laststring

			if ldirectory.count = 0 then
				last_success := false
			else
				last_success := true
			end
		ensure
			done: -- last_success implies last_directory set
		end

	change_current_directory (newdir: STRING) is
			-- Change current directory.
		do
			last_success := wrt.win_file_setcurdir (wrt.ew_string (newdir).to_c)
		end

	temporary_directory is
			-- Get temporary directory.
		do
			wrt.win_file_tmpdir
			ldirectory := wrt.win_laststring

			if ldirectory.count = 0 then
				last_success := false
			else
				last_success := true
			end
		end

feature -- File
	
	is_file (fname: STRING): BOOLEAN is
			-- Does the file `fname' exists?
		require
			valid_str: fname /= Void
		do
			Result := wrt.win_file_exists (wrt.ew_string (fname).to_c)
		end

	delete_file (fname: STRING) is
			-- Remove file `fname'.
		require
			valid_str: fname /= Void
			exists: is_file (fname)
			not_used: -- not is_open (fname)
		do
			last_success := wrt.win_file_remove (wrt.ew_string (fname).to_c)
		ensure
			done: last_success implies not is_file (fname)
		end

	copy_file (orig_file, dest_file: STRING) is
			-- Copy `orig_file' to `dest_file'.
		require
			valid_str: orig_file /= Void and dest_file /= Void
			exists: is_file (orig_file)
			no_existing_dest: not is_file (dest_file)
		do
			last_success := wrt.win_file_copy (wrt.ew_string (orig_file).to_c, wrt.ew_string (dest_file).to_c)
		ensure
			done: last_success implies is_file (dest_file)
		end
	
	move_file (orig_file, dest_file: STRING) is
			-- Move `orig_file' to `dest_file'.
		require
			valid_str: orig_file /= Void and dest_file /= Void
			exists: is_file (orig_file)
			no_existing_dest: not is_file (dest_file)
			not_used: -- not is_open (orig_name)
		do
			last_success := wrt.win_file_move (wrt.ew_string (orig_file).to_c, wrt.ew_string (dest_file).to_c)
		ensure
			done: last_success implies (is_file (dest_file) and not is_file (orig_file))
		end

feature {NONE}
	
	lcontent: P_LIST[STRING]
	ldirectory: STRING
	isize: INTEGER

feature -- Status
	
	last_success: BOOLEAN;
			-- Was the last operation successful?

	last_size: INTEGER is
			-- Result of last `free_disk_space' operation.
		require
			ok: last_success = true
		do
			Result := isize
		ensure
			done: Result >= 0
		end

	last_directory: STRING is
			-- Result of last system directory request such 
			-- as `current_directory'.
		require
			ok: last_success = true
		do
			Result := ldirectory
		ensure
			done: Result /= Void
		end

	last_content: P_LIST[STRING] is
			-- Result of last of `directory_content'.
		require
			ok: last_success = true
		do
			Result := lcontent
		ensure
			done: Result /= Void
		end

	last_error: INTEGER is
			-- Operating system last error code.
		require
			failed: last_success = false
		do
			Result := wrt.win_last_error
		end

end -- class F_FILE_SYSTEM
