-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, System
	description: "Memory-mapped file (read-only)"
	interface: client

class F_MEMORY_FILE

inherit
	F_MEMORY
		rename
			make as old_make
		redefine
			dispose, is_writable
		end

creation
	make

feature {CREATION_NONE}
	
	make (file_name: STRING) is
		require
			ok: file_name /= Void
			file_ok: -- fail if file is not valid and accessible 
		local
			filehandle: INTEGER 
		do
			finalisation_register
			size := 0
			
			-- open file handle
			filehandle := wrt.win_file_open (wrt.ew_string (file_name).to_c,
					wrt.Open_existing, true, false, true, false, -- read, write, sharer, sharew
					false) -- random access

			-- if succesful, map whole file
			if filehandle /= wrt.Invalid_handle_value then
				to_windows := wrt.win_mmap_file (filehandle)
				-- if mapping succesful, 
				if wrt.win_lastbool1 then
					size := wrt.win_file_size (filehandle)
					if size = 0 then -- useless empty file
						wrt.win_mmap_free (to_windows)
					end
				end
				wrt.win_file_close (filehandle)
			end
		ensure
			done: -- is_alive set if succesful
		end

feature
	
	is_writable: BOOLEAN is
			-- Memory mapped file read-only.
		do
			Result := false
		ensure then
			mmap_read_only: Result = false
		end

	is_alive: BOOLEAN is
			-- Is this memory mapped file working?
		do
			Result := size /= 0
		ensure
			size_ok: is_alive implies size > 0
		end

feature {NONE}
	
	dispose is
			-- Free memory mapped file.
		do
			if size /= 0 then
				win_mmap_free (to_windows)
				size := 0
			end
		end

end -- class F_MEMORY_FILE
