-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, System, Synchronisation
	description: "Mutex"
	interface: client, inheritance

class F_MUTEX

inherit
	F_SYNCH_OBJECT
		redefine
			internal_when_signaled
		end

creation
	make, make_existing

feature -- Status

	is_owned: BOOLEAN;
			-- Is the current thread owning this mutex?

feature {CREATION_NONE} -- Creation
	
	make (name: STRING) is
			-- Create new mutex object or open existing.
		require
			ok: name /= Void and then is_valid_name (name)
		do
			finalisation_register
			set_handle (wrt.win_mutex_create (wrt.ew_string (name).to_c))
			is_owned := false
		ensure
			not_owned: is_valid implies not is_owned
		end

	make_existing (name: STRING) is
			-- Open existing mutex.
		require
			ok: name /= Void and then is_valid_name (name)
		do
			finalisation_register
			set_handle (wrt.win_mutex_open (wrt.ew_string (name).to_c))
			is_owned := false
		ensure
			not_owned: is_valid implies not is_owned
		end
	
feature -- Change state
	
	release is
			-- Release mutex.
		require
			valid: is_valid
			got_it: is_owned
		do
			wrt.win_mutex_release (handle)
			is_owned := false
		end

feature {SYNCH_NOTIFIER}
	
	internal_when_signaled is
			-- Event
		do
			is_owned := true
			when_signaled
		end

end -- class F_MUTEX
