-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, System
	description: "Abstract synchronous stream"
	interface: abstract

deferred class F_STREAM

feature -- Constant

	Byte_maximum: INTEGER is 255

feature -- Access

	connect is
			-- Connect the stream so that it may, if the connect is successful, 
			-- be accessible.
		deferred
		ensure
			can_fail: -- success implies is_alive
		end

	imperative_connect is
			-- Connect the stream and generate an exception if connection fails.
		deferred
		ensure
			valid: is_alive
		end

	disconnect is
			-- Disconnect a valid stream.
		require
			valid: is_alive
		deferred
		ensure
			done: not is_alive
		end

feature -- Status
	
	is_alive: BOOLEAN is
			-- Is the stream connection alive (allowing read/write)?
		deferred
		end

	has_data: BOOLEAN is
			-- Is at least one byte waiting to be read?
		require
			valid: is_alive
		deferred
		end

	has_error: BOOLEAN is
			-- Is the stream in an error state?
		deferred
		end

	is_readable: BOOLEAN is
			-- Is the stream readable?
		deferred
		end

	is_writable: BOOLEAN is
			-- Is the stream writable?
		deferred
		end

feature -- Read/write
	
	last_byte: INTEGER is
			-- Last byte read.
		deferred
		ensure
			byte_size: Result >= 0 and then Result <= Byte_maximum
		end
	
	read_byte is
			-- Read one byte and update last byte.
		require
			valid: is_alive
			readable: is_readable
			available: has_data
		deferred
		end

	write_byte (byte: INTEGER) is
			-- Write a byte.
		require
			valid: is_alive
			writable: is_writable
			byte_size: byte >= 0 and then byte <= Byte_maximum 
		deferred
		end

end -- class F_STREAM
