-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, System, Synchronisation
	description: "Base synchronisation event (from operating system)"
	interface: abstract, inheritance

class F_SYNCH_OBJECT

inherit
	WIN_OBJECT
		end
	WIN_API
		end
	WIN_RUNTIME_FINAL
		end

feature {NONE} -- Attribute(s)

	command: GUI_COMMAND

feature -- Notifier
	
	notifier: SYNCH_NOTIFIER is
			-- Synchronisation notifier.
		once
			!!Result.make
		end

feature -- Status

	is_valid: BOOLEAN is
			-- Is this object valid?
		do
			Result := is_valid_handle
		end

	is_valid_name (nm: STRING): BOOLEAN is
			-- Is this string a valid event object name?
		require
			ok: nm /= Void
		local
			i: INTEGER
		do
			if nm.count > 0 and nm.count <= wrt.Max_path then
				Result := true
				from i := nm.count variant i until i = 0 or not Result loop
					if nm.item (i) = '\' then
						Result := false
					end
					i := i - 1
				end
			end
		ensure
			done: -- Result = (0 < nm.count < MAX_PATH and all nm.item /= '\')
		end

feature
	
	synch_wait is
			-- Synchronous wait for event.
		require
			valid: is_valid
		do
			if wrt.win_synch_wait (handle, wrt.Infinite) then
				internal_when_signaled
				--? abandoned mutex
			end
		end

	finite_wait (delay_ms: INTEGER) is
			-- Synchronous wait for event during given delay.
		require
			ok: delay_ms >= 0
			valid: is_valid
		do
			if wrt.win_synch_wait (handle, delay_ms) then
				internal_when_signaled
			end
		end

	wait is
			-- Asynchronous non-blocking wait.
		require
			valid: is_valid
		do
			notifier.register_synch_object (Current)
		end

	close is
			-- Close event object.
		require
			valid: is_valid
		do
			dispose
		ensure
			done: not is_valid
		end

feature -- Command(s)

	set_command (cmd: GUI_COMMAND) is
			-- Associate command with this event.
		require
			may_be_void: -- Void to cancel existing command
		do
			command := cmd
		ensure
			keep_reference: command = cmd
		end

feature {PUBLIC_NONE} -- Event(s)
	
	when_signaled is
			-- Event has been signaled.
			-- Default: execute associated command if any.
		do
			if command /= Void then
				command.execute
			end
		end

feature {SYNCH_NOTIFIER}
	
	internal_when_signaled is
			-- Internal signal event.
		do
			when_signaled
		end

feature {NONE}
	
	dispose is
			-- Object has been finalised.
		do
			if handle /= Invalid_handle then
				win_file_close (handle)
				handle := Invalid_handle
			end
		end

end -- class F_SYNCH_OBJECT
