-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, System
	description: "Text file access (a text file is a set of lines)"
	interface: client

class F_TEXTFILE

inherit
	F_FILE
		rename
			make as file_make
		export
			{NONE} file_make
		end

creation
	make

feature {CREATION_NONE} -- Creation.

	make is
			-- Create textfile.
		do
			file_make
			reset_textmode
		end

feature -- Mode

	reset_textmode, set_textmode_standard is
			-- Both <cr><lf> and <lf> are separators, lonely <cr> are ignored
		do
			set_textmode_character ('%N')
			textmode_crlf := true
		end

	set_textmode_dos is
			-- <cr><lf> is the only separator. both lonely <cr> and <lf> are 
			-- ignored.
		do
			textmode_crlf := true
			textmode_char := false
		end

	set_textmode_unix is
			-- <lf> are separators.
		do
			set_textmode_character ('%N')
		end

	set_textmode_character (chr: CHARACTER) is
			-- The above character is the sole separator.
		do
			textmode_crlf := false
			textmode_char := true
			separator_char := chr
		end

feature -- Simple character access

	last_character: CHARACTER is
			-- Last character read.
		do
			Result := wrt.int_to_char (last_byte)
		end

	read_character is
			-- Read a character.
		require
			readable: is_readable
			valid: is_alive
			data: has_data
		do
			read_byte
		end

	write_character (c: CHARACTER) is
			-- Write a character.
		require
			writable: is_writable
			valid: is_alive
		do
			write_byte (c.code)
		end

feature {NONE}
	
	textmode_char: BOOLEAN
	separator_char: CHARACTER
	textmode_crlf: BOOLEAN

feature -- Read/write line

	last_string: STRING;
			-- Last read string (fresh object for each read_string).

	read_string is
			-- Read the string into last_string. Separator char(s) are not put at 
			-- the end of the string.
		require
			valid: is_alive
			readable: is_readable
			data: has_data
		local
			ch: CHARACTER
			stop: BOOLEAN
			wascr: BOOLEAN
		do
			!!last_string.make (0)

			from
				wascr := false
				stop := false
			until
				stop or else not has_data
			loop
				read_character
				ch := last_character
					
				if textmode_crlf then -- crlf
					if wascr = true then
						wascr := false
						if ch = '%N' then
							stop := true
						else
							last_string.append_character ('%R')
						end
					end
					if ch = '%R' then
						wascr := true
					end
				end
				if textmode_char then -- is it the separator?
					if ch = separator_char then
						stop := true
					end
				end

				if not stop and not wascr then
					last_string.append_character (ch)
				end
			end -- loop

			if wascr then
				last_string.append_character ('%R')
			end
		end
	
	write_string (ostr: STRING) is
			-- Write a line to the file (a separator character is 
			-- automatically added).
		require
			valid_string: ostr /= Void
			valid_file: is_alive
			writable: is_writable
		local
			i: INTEGER
			str: STRING
		do
			!!str.make (ostr.count + 5)
			str.copy (ostr)

			-- add end of line.
			if textmode_crlf then
				str.append_string ("%R%N")
			else
				if textmode_char then
					str.append_character (separator_char)
				end
			end

			-- write line.
			from
				i := 1
			invariant
				valid_string: not str.empty
			variant
				str.count - i + 1
			until
				i > str.count
			loop
				write_character (str.item (i))
				i := i + 1
			end
			
		end

invariant
	valid_separator: textmode_crlf or textmode_char
	valid_char: -- Character_bits = 8

end -- class F_TEXTFILE

