-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, System
	description: "System timer"
	interface: client, inheritance

class F_TIMER

inherit
	COMMON_NOTIFIER
		end

creation
	make

feature {CREATION_NONE} -- Creation

	make is
			-- Create timer.
		do
			is_active := false
			set_id (0)
			set_delay (Default_delay)
			command := Void
		end

feature -- Status
	
	is_active: BOOLEAN;
			-- Is the timer active?
	
	set_delay (ms: INTEGER) is
			-- Set the requested delay (in milliseconds) between executions 
			-- of this timer. This is a wish, actual delay between calls may 
			-- vary and can be greater or equal to 'delay'. This delay should 
			-- preferably not be too small (>100 ms).
		require
			down: not is_active
		do
			delay := ms
		ensure
			done: delay = ms
		end

	set_command (com: GUI_COMMAND) is
			-- Set callback command.
		require
			down: not is_active
			void: -- void means no command
		do
			command := com
		ensure
			done: command = com
		end
		
	start is
			-- Lauch the timer.
		require
			stopped: not is_active
		do
			notifier.register_timer (Current)
			is_active := true
		end

	stop is
			-- Stop the timer.
		require
			running: is_active
		do
			notifier.unregister_timer (Current)
			is_active := false
			id := 0
		end

feature {NONE}
	
	command: GUI_COMMAND
	Default_delay: INTEGER is 1000 -- 1 second

feature {NOTIFIER}
	
	delay: INTEGER
	id: INTEGER

	set_id (newid: INTEGER) is
		do
			id := newid
		end

	internal_when_executed is
		do
			when_executed
		end

feature {PUBLIC_NONE}
	
	when_executed is
			-- Callback every 'delay' milliseconds or so.
			-- Default behaviour: call command.
		require
			running: is_active
		do
			if command /= Void then
				command.execute
			end
		end

end -- class F_TIMER
