-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Windows
	description: "Child window"
	interface: inheritance

class CHILD_WINDOW

inherit
	WINDOW
		redefine
			register, when_hscroll, when_vscroll
		end
	
creation
	make_child, make_child_scroll, 
	make_child_vertical_scroll, 
	make_child_horizontal_scroll

creation {STD_CONTROL}
	make_handle
	
feature {CREATION_NONE} -- Creation

	make_child (par: like parent) is
			-- Create a child window.
		require
			valid: par /= Void and then par.is_valid
		do
			-- do it
			make_internal (par, false, false)
		ensure
			keep_reference: par = parent
		end

	make_child_horizontal_scroll (par: like parent) is
			-- Make a child window with a horizontal scrollbar.
		require
			valid: par /= Void and then par.is_valid
		do
			make_internal (par, true, false)
		ensure
			keep_reference: par = parent
		end

	make_child_vertical_scroll (par: like parent) is
			-- Make a child window with a vertical scrollbar.
		require
			valid: par /= Void and then par.is_valid
		do
			make_internal (par, false, true)
		ensure
			keep_reference: par = parent
		end
	
	make_child_scroll (par: like parent) is
			-- Make a child window with both vertical and horizontal scrollbar.
		require
			valid: par /= Void and then par.is_valid
		do
			make_internal (par, true, true)
		ensure
			keep_reference: par = parent
		end

feature {NONE}
	
	make_handle (par: WINDOW; hdle: INTEGER) is
			-- Creation
		do
			parent := par
			!!wo.make_handle (hdle)
			initialise_window -- register surrogate for STD_CONTROL windows.
			parent.add_child (Current)
		ensure
			keep_reference: par = parent
		end

feature {NONE} -- Cursor

	free_cursor: BOOLEAN is
			-- Has customisable cursor?
		do
			Result := false
		end

feature {NONE} -- Creation

	make_internal (par: like parent ; hslid, vslid: BOOLEAN) is
		require
			valid: par /= Void and then par.is_valid
		do
			-- create
			parent := par
			!!wo.make_handle (wrt.win_create_child (parent.handle, free_cursor, hslid, vslid))
			has_horizontal_scrollbar := hslid
			has_vertical_scrollbar := vslid
			register
		end

	register, frozen child_register is
		do
			-- register with parent
			parent.add_child (Current)

			-- scrollbars
			if has_horizontal_scrollbar = true then
				!WINDOW_SCROLLBAR!horizontal_scrollbar.make_horizontal (Current)
			end
			if has_vertical_scrollbar = true then
				!WINDOW_SCROLLBAR!vertical_scrollbar.make_vertical (Current)
			end
			
			window_register
		end

	has_horizontal_scrollbar, has_vertical_scrollbar: BOOLEAN --? useful

feature -- Size

	size: RECTANGLE is
			-- Return the window external size relative to the client area 
			-- of the parent window for child windows.
		do
			wrt.win_size (handle, parent.handle)
			Result := make_size (wrt.win_lastint1, wrt.win_lastint2, wrt.win_lastint3, wrt.win_lastint4) 
		end
	
feature -- Attached scrollbars

	horizontal_scrollbar: WINDOW_SCROLLBAR
	vertical_scrollbar: WINDOW_SCROLLBAR

feature {PUBLIC_NONE, WINDOW_SCROLLBAR} -- Scrollbar events --? export

	when_horizontal_scrolled (req_pos: INTEGER) is
			-- Horizontal scrollbar scrolled, default: set slider position.
			-- Actual position can be obtained from horizontal_scrollbar.slider_position
		require
			valid_sbar: horizontal_scrollbar /= Void
		do
			horizontal_scrollbar.set_slider_position (req_pos)
		end

	when_vertical_scrolled (req_pos: INTEGER) is
			-- Vertical scrollbar scrolled, default: set slider position.
			-- Actual position can be obtained from vertical_scrollbar.slider_position
		require
			valid_sbar: vertical_scrollbar /= Void
		do
			vertical_scrollbar.set_slider_position (req_pos)
		end

feature -- Style

	set_border (hasborder: BOOLEAN) is
			-- Set the window's border style:
			-- hasborder = true implies one pixel border, otherwise not.
		require
			valid: is_valid
		do
			wrt.win_border (handle, hasborder)
		end

feature -- Siblings
	
	is_sibling (other: WINDOW): BOOLEAN is
			-- Is other an _older_ sibling of us? 
			-- (exported for preconditions)
		require
			valid_window: other /= Void
			has_parent: parent /= Void
		local
			it: P_ITERATOR[CHILD_WINDOW]
			found_us: BOOLEAN
		do
			check 
				parent: parent /= Void
				parent_has_children: parent.child_list /= Void
			end

			from
				it := parent.child_list.iterator
				it.first
			variant
				it.forward_variant
			until
				it.outside
			loop
				if it.item = Current then
					found_us := true
				end
				if it.item = other then
					if not found_us then
						Result := true -- other is before Current
					end
				end
				-- next step
				it.forth
			end

			if not found_us then
				Result := false -- not sister, not even child
			end
		end

	next: CHILD_WINDOW is
			-- Next sibling in the cycle (may be Current).
		local
			cl: P_LIST[CHILD_WINDOW]
			it: P_ITERATOR[CHILD_WINDOW]
		do
			cl := parent.child_list
			
			cl.equality_search (Current)
			check good_parent: cl.found end
			
			it := cl.iterator
			it.synchronize
			it.forth
			if it.outside then -- last
				it.first
			end
			
			Result := it.item
		end

	previous: CHILD_WINDOW is
			-- Previous sibling in the cycle (may be Current).
		local
			cl: P_LIST[CHILD_WINDOW]
			it: P_ITERATOR[CHILD_WINDOW]
		do
			cl := parent.child_list
			
			cl.equality_search (Current)
			check good_parent: cl.found end
			
			it := cl.iterator
			it.synchronize
			it.back
			if it.outside then -- first
				it.last
			end
			
			Result := it.item
		end

feature -- Setting up resize rules
	
	size_follows_parent is
			-- Child window fills parent client area.
		do
			parent_sized := true
			enable_resize
		end

	fixed_width is
			-- Width remains constant when layout changed.
		require
			valid: is_valid
		do
			constant_width := true
		end

	fixed_height is
			-- Height remains constant when layout changed.
		require
			valid: is_valid
		do
			constant_height := true
		end

feature {WINDOW} -- Event(s)

	when_parent_resized is
			-- Called by parent window when it is itself resized.
		local
			r: RECTANGLE
			low, up: POINT
			newd: INTEGER
		do
			check icon: not is_iconic end
			-- dynamic resizing

			if resize_this then --? and then is_visible
				r := size
				low := r.lower
				up := r.upper

				-- follow parent size
				if parent_sized then
					-- left
					newd := parent.client_size.upper.x
					if newd < low.x then
						low.set_x (newd)
					end
					up.set_x (newd)

					-- down
					newd := parent.client_size.upper.y
					if newd < low.y then
						low.set_y (newd)
					end
					up.set_y (newd)

					-- up
					if up.y < 0 then
						up.set_y (0)
					end
					low.set_y (0)

					-- right
					if up.x < 0 then
						up.set_x (0)
					end
					low.set_x (0)
				end

				-- fixed dimensions
				if constant_width then
					if low.x = r.lower.x then -- up changed
						low.set_x ((up.x - r.width + 1).max (0))
					else -- low changed
						up.set_x (low.x + r.width - 1)
					end
					check cons_width: low.x /= 0 implies (up.x - low.x + 1 = r.width) end
				end
				if constant_height then
					if low.x = r.lower.x then -- up changed
						low.set_y ((up.y - r.height + 1).max (0))
					else -- low changed
						up.set_y (low.y + r.height - 1)
					end
					check cons_height: low.y /= 0 implies (up.y - low.y + 1 <= r.height) end
				end
				
				check 
					order_x: low.x <= up.x
					order_y: low.y <= up.y
				end

				!!r.make_points (low, up)

				if not r.is_equal (size) then
					set_size (r) -- will call 'when_resized'
				end
			end
		end

feature {NONE}
	
	when_hscroll (code, position: INTEGER) is
		require else
			redefined_and_bar: horizontal_scrollbar /= Void
		do
			horizontal_scrollbar.when_scroll (code, position)
		end

	when_vscroll (code, position: INTEGER) is
		require else
			redefined_and_bar: vertical_scrollbar /= Void
		do
			vertical_scrollbar.when_scroll (code, position)
		end

feature {NONE}
	
	enable_resize is
			-- enable when_parent_resize
		do
			resize_this := true
			parent.set_dynamic_child_resize (true)
			--? really call dynamic resize now?
			when_parent_resized
		end

	resize_this, parent_sized: BOOLEAN
	constant_height, constant_width: BOOLEAN

invariant
	has_parent: parent /= Void
	-- hscroll: has_horizontal_scrollbar implies horizontal_scrollbar /= Void
	-- vscroll: has_horizontal_scrollbar implies vertical_scrollbar /= Void

end -- class CHILD_WINDOW
