-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Windows
	description: "Menu item and associated action"
	interface: inheritance, client

class MENU_ITEM

inherit
	WIN_API
		end

creation
	make

feature {OVERLAPPED_WINDOW, MENU} -- Atttribute(s)
	
	id: INTEGER
		-- Unique id.

	parent_menu: MENU
		-- Parent menu of the item.

feature {NONE} -- Private attributes

	command: GUI_COMMAND
			-- Optional execution command.

	pre_command: GUI_COMMAND
			-- Optional pre-selection command.

	leader: HEAD_MENU_ITEM
			-- Optional leader.
						
	accelerator: ACCELERATOR
			-- Optional associate accelerator.

feature {MENU}
					
	name: STRING
			-- Menu name (displayed by the Application).

	help: STRING
			-- Help string.

feature {OVERLAPPED_WINDOW, MENU}
	
	set_id (newid: INTEGER) is
			-- Set item's ID.
		require
			pre_attach: parent_menu /= Void
			good_id: newid /= 0
		do
			id := newid
		ensure
			done_id: id = newid
			attached: is_valid
		end

	reset_id is
		do
			id := 0
		ensure
			done: id = 0
			detached: not is_valid
		end

	set_parent (newparent: MENU) is
			-- Set item's parent.
		require
			parent_ok: newparent /= Void
		do
			parent_menu := newparent
		ensure
			done_parent: parent_menu = newparent
		end

feature {CREATION_NONE} -- Creation
	
	make (nm: STRING) is
			-- Create the item & attach it to a popup menu.
		require
			valid_string: nm /= Void
		do
			!!name.make (0)
			name.copy (nm)
			!!help.make (0)
			leader := Void
		ensure
			not_attached: not is_valid
		end

feature -- Attributes

	is_valid: BOOLEAN is
			-- Is this item valid and attached to a parent menu?
		do
			Result := id /= 0 and parent_menu /= Void
		end

	enabled: BOOLEAN is
			-- Is this item enabled or disabled?
		require
			valid: is_valid
		do
			Result := wrt.win_menu_isenabled (parent_menu.handle, id)
		end

	is_checked: BOOLEAN is
			-- Does this item have a visible check mark?
		require
			valid: is_valid
		do
			Result := wrt.win_menu_ischecked (parent_menu.handle, id)
		end

feature -- Update
	
	set_name (s: STRING) is
			-- Set name. It will not be modified in the actual 
			-- after the parent MENU has been attached.
		require
			valid_string: s /= Void
			valid_menu: is_valid
		do
			name.copy (s)
			wrt.win_menu_chtext (parent_menu.handle, id, wrt.ew_string (s).to_c)
		end

	set_help (hs: STRING) is
			-- Set the optional help string associated with this item.
		require
			valid_string: hs /= Void
		do
			help.copy (hs)
		end

	set_command (gc: GUI_COMMAND) is
			-- Optional selection command (if when_selected not redefined).
		require
			nothing: -- Void means no callback command
		do
			command := gc
		ensure
			keep_reference: command = gc
		end

	set_prepare_command (gc: GUI_COMMAND) is
			-- Optional preparation command (if when_prepared not redefined).
		require
			nothing: -- Void means no callback command
		do
			pre_command := gc
		ensure
			keep_reference: pre_command = gc
		end

	set_leader (lead: HEAD_MENU_ITEM) is
			-- Set optional leading menu item for radio-type items.
		require
			valid_lead: lead /= Void
			not_leader: lead /= Current
		do
			leader := lead
			leader.add_item (Current)
		ensure
			keep_reference: lead = leader
		end

feature -- Setup
	
	enable is
			-- Enable menu item.
		require
			valid: is_valid
		do
			wrt.win_menu_enable (parent_menu.handle, id, true)
		ensure
			done: enabled
		end
	
	disable is
			-- Grey out menu item.
		require
			valid: is_valid
		do
			wrt.win_menu_enable (parent_menu.handle, id, false)
		ensure
			done: not enabled
		end

	check_mark is
			-- Set check mark.
		require
			valid: is_valid
		do
			if not is_checked then
				if leader /= Void then
					leader.uncheck_all
				end
			
				--? execute
				wrt.win_menu_checked (parent_menu.handle, id, true)
			end
		ensure
			done: is_checked
		end

	uncheck_mark is
			-- Unset check mark.
		require
			valid: is_valid
		do
			wrt.win_menu_checked (parent_menu.handle, id, false)
		ensure
			done: not is_checked
		end

	detach is
			-- Remove this item from its menu.
		require
			attached: is_valid
		do
			if leader /= Void then
				leader.remove_item (Current)
				leader := Void
			end
			wrt.win_menu_delete (parent_menu.handle, id)
			parent_menu.remove_item (Current)
			parent_menu := Void
			id := 0
		ensure
			detached: not is_valid
		end

feature -- Accelerator

	has_accelerator: BOOLEAN is
			-- Has this item got an associated keyboard accelerator?
		do
			Result := accelerator /= Void
		end

	set_accelerator (newacc: ACCELERATOR) is
			-- Set associated keyboard accelerator.
		require
			ok: newacc /= Void
		do
			accelerator := newacc
		ensure
			keep_reference: accelerator = newacc
			done: has_accelerator
		end
	
feature {WIN_API} -- Accelerator system interface
	
	accelerator_to_windows: F_MEMORY is
			-- Low level keyboard accelerator entry.
		require
			has_accel: has_accelerator
			valid: is_valid
		do
			Result := accelerator.to_windows (id)
		ensure
			is_copy: Result /= Void
		end

feature {PUBLIC_NONE} -- Event(s)
	
	when_selected is
			-- Item selected for execution. 
			-- Default behaviour: calls the associated command if there
			-- is one. If there is a leader, check this item and uncheck 
			-- the (single) associated button which is checked.
		require
			valid: is_valid
			is_enabled: enabled
		do
			if leader /= Void then
				if not is_checked then
					check_mark
				end
			end
			if command /= Void then
				command.execute
			end
		end

	when_touched is
			-- Item is being selected by the user. 
			-- Default processing: send a help string message to the 
			-- parent window.
		do
			if not help.empty then
				parent_menu.parent.set_menu_help (help)
			end
		end

	when_prepared is
			-- Item is about to be displayed.
			-- An application can use this event to change the state 
			-- of the item prior to display (eg. disable the item if the 
			-- corresponding data is not available currently).
			-- Default behaviour: calls the associated command if
			-- available.
		require
			valid: is_valid
		do
			if pre_command /= Void then
				pre_command.execute
			end
		end

feature -- Action
	
	execute is
			-- Simulate selection of menu item.
		do
			when_selected
		end

feature {OVERLAPPED_WINDOW} -- Selected event redirection.
	
	internal_when_touched is
		do
			when_touched
		end

feature {MENU} -- Preparation event.

	internal_when_prepared is
		do
			when_prepared
		end

end -- class MENU_ITEM
