-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Windows
	description: "Mouse handler for one button (associated with WINDOW)"
	interface: inheritance

class MOUSE_HANDLER

inherit
	WIN_API
		end

creation
	make

feature {CREATION_NONE} -- Creation
	
	make (win: like parent) is
			-- Create the mouse handler for one button.
		do
			parent := win
			!!point.make
		ensure
			down: not is_down and not dragging
			keep_reference: win = parent
		end

feature -- Parent window
	
	parent: WINDOW
			-- Associated window.

feature -- Mouse button status

	is_down: BOOLEAN
			-- Is the button down?
						
	point: POINT
			-- Current position of the mouse pointer.

feature {NONE}
	
	dragging: BOOLEAN

feature {PUBLIC_NONE} -- Events

	when_down is
			-- Mouse button went down.
		require
			down: is_down
		do 
		end
			
	when_up is
			-- Mouse button went up.
		require
			up: not is_down
		do 
		end

	when_moved is
			-- Mouse moved.
		do 
		end

	when_dragged is
			-- Mouse moved and button down.
		require
			down: is_down
		do
		end
	
	when_drag_started is
			-- Starting drag.
		require
			down: is_down
		do
		end

	when_drag_finished is
			-- Finished drag
		require
			up: not is_down
		do
		end

feature {WINDOW} -- Events from window
	
	internal_when_down is
		do
			if dragging then -- remains from last `down' event without `up'...
				is_down := false
				dragging := false
				wrt.win_endcapture
				when_drag_finished
				when_up -- simulated up.
			end
			
			is_down := true
			wrt.win_capture (parent.handle)
			
			when_down
		ensure
			ok: is_down and not dragging
		end

	internal_when_up is
		do
			is_down := false
			wrt.win_endcapture
			
			if dragging then
				dragging := false
				when_drag_finished
			end
			
			when_up
		ensure
			ok: not is_down and not dragging
		end

	internal_when_moved (p: POINT) is
		require
			ok: p /= Void
		do
			set_point (p)
			when_moved
			if is_down then
				if not dragging then
					when_drag_started
					dragging := true
				end
				when_dragged
			end
		ensure
			drag: is_down implies dragging
		end

feature {NONE} -- (Internal) set point

	set_point (p: POINT) is
			-- Set current mouse point
		do
			point.copy (p)
		end

invariant
	valid_point: point /= Void

end -- class MOUSE_HANDLER
