-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Windows
	description: "Independent overlapping window"
	interface: inheritance

class OVERLAPPED_WINDOW

inherit
	WINDOW
		redefine
			parent, when_menu_selected, 
			when_menu_prepared, when_menu_touched
		end

creation
	make_overlapped

feature {CREATION_NONE} -- Creation

	make_overlapped (par: like parent) is
			-- Create an overlapping window (the parent, which must be an 
			-- overlapped window, is mainly used only to communicate)
		require
			valid: par /= Void
		do
			parent := par
			!!wo.make_handle (wrt.win_create)
			register
		ensure
			ok: is_valid
			keep_reference: par = parent
		end

feature -- Parent

	parent: OVERLAPPED_WINDOW

feature -- Window's size

	size: RECTANGLE is
			-- Return the window external size, in screen coordinates 
			-- for an overlapped window.
		do
			wrt.win_size (handle, 0)
			Result := make_size (wrt.win_lastint1, wrt.win_lastint2, wrt.win_lastint3, wrt.win_lastint4)
		end

feature -- Actions: iconisation.

	maximize is
			-- Maximise (show full screen) the window.
		require
			valid: is_valid
		do
			wrt.win_show (handle, wrt.Sw_showmaximized)
		ensure
			not_iconic: is_iconic = false
		end

	minimize is
			-- Minimise (reduce to an icon) the window.
		require
			valid: is_valid
		do
			wrt.win_close (handle)
		ensure
			iconic: is_iconic
		end

	restore is
			-- Show the window and restore to its normal size if
			-- previously minimised or maximised.
		require
			valid: is_valid
		do
			wrt.win_show (handle, wrt.Sw_shownormal)
		end
	
feature -- Keyboard accelerators

	apply_accelerators is
			-- Apply keyboard accelerators for items of menus CURRENTLY 
			-- associated with this window, it should be called after the 
			-- menu system has been set up to enable keyboard shortcuts.
			-- WARNING: If the menus are updated (or at least items with 
			-- associated accelerators are added or removed), this procedure 
			-- should be called again to update the table maintained by 
			-- the system.
		require
			valid: is_valid
		do
			if not menuitems.is_empty then
				-- check previous table
				if accel /= Void then
					accel.force_dispose
				end

				-- new one
				!!accel.make (menuitems.to_item_list)
				wrt.win_accel_enable (handle, accel.handle) --? when_closed, disable
			end
		end

feature {NONE} -- Accelerators table

	accel: WIN_ACCEL
	--? should be disposed when window closed?

feature {MENU} -- Menu bar
	
	set_menu (menu: MENU_BAR) is
			-- Setup menu bar.
		require
			valid: is_valid
			menu_valid: menu /= Void
		do
			wrt.win_menu_set (menu.handle, handle)
		end

feature {PUBLIC_NONE} -- Event(s)
	
	when_menu_help_changed (menu_text: STRING) is
			-- Context menu help changed.
		require
			valid_string: menu_text /= Void
		do
		end

	when_menu_help_reset is
			-- Reset context menu help.
		do
		end

feature {MENU_ITEM}
	
	set_menu_help (hlp_str: STRING) is
			-- Menu help changed.
		do
			when_menu_help_changed (hlp_str)
		end

feature {NONE} -- Menu management events
	
	when_menu_selected (id: INTEGER) is
			-- (Internal) menu selection
		require else
			exists: is_valid_menu_id (id)
		do
			check hasitems: menuitems /= Void end
			
			menuitems.key_search (id)
			if menuitems.found then
				menuitems.item.execute
			end -- else exception?
		end

	when_menu_prepared (menu_handle: INTEGER) is
			-- (Internal) menu prepared
		require else
			effected: true
		do
			if menus /= Void then
				menus.key_search (menu_handle)
				if menus.found then -- maybe not if popup has no live items or system menu
					menus.item.when_prepared
				end
			end
		end

	when_menu_touched (id: INTEGER; reset: BOOLEAN) is
			-- (Internal) menu touched, for help
		require else
			effected: true
		do
			if reset then
				when_menu_help_reset
			else
				check hasitems: menuitems /= Void end

				menuitems.key_search (id)
				if menuitems.found then
					menuitems.item.internal_when_touched
				end
			end
		end

feature {NONE} -- (Internal) menu management
	
	Maximum_menu_id: INTEGER is 10000

	menuitems: P_DICTIONARY[MENU_ITEM, INTEGER] -- int = ID
	menus: P_DICTIONARY[MENU, INTEGER] -- int = handle

feature {MENU} -- List of items management

	register_menu_item (item: MENU_ITEM) is
			-- Register menu item for callbacks
		require
			valid_item: item /= Void
		local
			clock: F_CLOCK
			random: P_RANDOM
			i, candidate_id: INTEGER
		do
			-- add menu
			if not is_valid_menu (item.parent_menu.handle) then
				if menus = Void then
					!!menus.make
				end
				menus.add (item.parent_menu, item.parent_menu.handle)
			end

			-- init random
			!!clock clock.get_tick
			!!random.make (clock.last_tick)

			if menuitems = Void then
				!!menuitems.make
			end

			-- find menuitem ID.
			from
				i := Maximum_menu_id
				item.reset_id
			variant i until item.id /= 0 or else i = 0 loop
				random.get_number (1, Maximum_menu_id)
				candidate_id := random.last_number
				check ok_id: candidate_id /= 0 end

				menuitems.key_search (candidate_id)
				if not menuitems.found then
					item.set_id (candidate_id)
				end
			
				i := i - 1
			end

			check done: i > 0 and item.id /= 0 end -- should not happen...
			
			-- register item
			menuitems.add (item, item.id)
		end

	unregister_menu_item (item: MENU_ITEM) is
		do
			menuitems.key_search (item.id)
			check found: menuitems.found end
			menuitems.remove
		end

	unregister_menu (menu: MENU) is
		do
			menus.key_search (menu.handle)
			check found: menus.found end
			menus.remove
		end

feature {NONE} -- Events support
	
	is_valid_menu_id (id: INTEGER): BOOLEAN is
			-- Check if menu ID exists.
		do
			check menuitems /= Void end
			menuitems.key_search (id)
			Result := menuitems.found
		end
			
	is_valid_menu (hdle: INTEGER): BOOLEAN is
			-- Check if menu with corresponding handle is known.
		do
			if menus /= Void then
				menus.key_search (hdle)
				Result := menus.found
			end
		end

end -- class OVERLAPPED_WINDOW
