-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Xcontrols
	description: "Status bar (help string at the bottom of a window)"
	comment: "the toolbar is positioned automatically" 
	interface: client

class STATUS_BAR

inherit
	CHILD_WINDOW
		rename
			make_child as make
		redefine
			when_created, when_drawn
		end

creation
	make

feature -- Interface

	set_status (txt: STRING) is
			-- Set status string.
		do
			current_string.copy (txt)
			if is_visible then
				paint
			end
		end

	reset_status is
			-- Reset status.
		do
			current_string.wipe_out
		end

	set_font (ft: FONT) is
			-- Set font.
		require
			ok: ft /= Void
		do
			device.set_font (ft)
			reset_size
			paint
		end

feature {NONE}
	
	current_string: STRING
	interspace: INTEGER is 4
	draw_top_line: BOOLEAN is true

feature {NONE}
	
	status_height: INTEGER is
		require
			not_ready: not device.is_ready
		local
			dt: DRAWABLE_TEXT
		do
			!!dt.make
			dt.set_text ("ABCabc123")
			
			device.prepare
			Result := device.text_extent (dt).height + 2 * interspace
			device.release
		end

	reset_size is
		local
			r: RECTANGLE
		do
			r := client_size
			r.set_height (status_height)
			set_client_size (r)
		end

feature {PUBLIC_NONE}

	when_created is
			-- Initialise.
		local
			cfg: F_CONFIG
		do
			!!current_string.make (0)
			
			-- set height
			fixed_height

			-- border
			set_border (false)

			-- set default font
			!!cfg
			device.set_font (cfg.default_dialog_font)

			-- height
			reset_size
		end

	when_drawn is
			-- Paint.
		local
			clr: RGB_COLOR
			gr: GRAPHICS_ROUTINES
			dr: DRAWABLE_RECTANGLE
			dl: DRAWABLE_LINE
			dt: DRAWABLE_TEXT
			pt: POINT
		do
			!!gr

			-- draw background
			!!clr.light_gray
			gr.set_pen_and_brush (device, clr)

			!!dr.make
			dr.set_position (client_size)
			device.draw (dr)
		
			-- optional top line
			if draw_top_line then
				!!pt.make
				!!dl.make

				-- white line above
				dl.set_start (client_size.lower)
				pt.set_x (client_size.upper.x)
				dl.set_final (pt)

				!!clr.black
				gr.set_pen (device, clr)

				device.draw (dl)
			end
			
			-- draw text
			!!dt.make
			dt.set_text (current_string)
			
			!!pt.make
			pt.set_x (interspace)
			pt.set_y (((client_size.height - device.text_extent (dt).height) // 2)-1)
			dt.set_position (pt)
			
			device.draw (dt)
		end

end -- class STATUS_BAR
	
