-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Xcontrols
	description: "Tool bar (aligned controls at the top of a window)"
	comment: "the toolbar is positioned automatically" 
	interface: client

class TOOLBAR

inherit
	CHILD_WINDOW
		rename
			make_child as make
		redefine
			when_created, when_drawn
		end
	DIALOG
		end

creation
	make

feature -- Default dialog implementation

	when_prepared is
		do
		end

	when_applied is
		do
		end

	when_finished is
		do
		end

feature -- Interface

	add_element (el: DIALOG_ELEMENT) is
			-- Add an element to the toolbar.
		require
			ok: el /= Void
		local
			r, cs: RECTANGLE
			pt: POINT
		do
			r := el.indep_size
			
			if current_position = Void then
				!!current_position.make
				current_position.set_xy (interspace, interspace)
			end

			-- position
			r.translate_lower (current_position)
			el.indep_set_size (r)
			
			-- prepare next position.
			current_position.set_x (current_position.x + r.width)
			
			-- height
			r.set_upper_y (r.upper.y + interspace)
			r := dialog_units.indep_to_abs (r)
			cs := client_size
			if r.upper.y > cs.upper.y then
				!!pt.make
				cs.set_lower (pt) -- (0, 0)
				pt.set_xy (cs.upper.x, r.upper.y)
				cs.set_upper (pt)
				set_client_size (cs)
			end
		end

	add_space is
			-- Add normalised space between elements.
		require
			not_first: -- current_position /= Void
		do
			check not_first: current_position /= Void end

			current_position.set_x (current_position.x+ 2*interspace)
		end

feature {NONE}
	
	current_position: POINT
	interspace: INTEGER is 2
	draw_frame: BOOLEAN is true

feature {PUBLIC_NONE}

	when_created, toolbar_when_created is
			-- Initialise.
		do
			make_dialog -- init DIALOG
			
			fixed_height

			set_border (false)
		end

	when_drawn is
			-- Paint background.
		local
			clr: RGB_COLOR
			gr: GRAPHICS_ROUTINES
			dr: DRAWABLE_RECTANGLE
			dl: DRAWABLE_LINE
			pt: POINT
		do
			!!gr

			-- background
			!!clr.light_gray
			gr.set_pen_and_brush (device, clr)

			!!dr.make
			dr.set_position (client_size)
			device.draw (dr)
			
			-- optional border
			if draw_frame then
				!!pt.make
				!!dl.make

				-- white line above
				dl.set_start (client_size.lower)
				pt.set_x (client_size.upper.x)
				dl.set_final (pt)

				!!clr.white
				gr.set_pen (device, clr)
				device.draw (dl)

				-- black line at the bottom
				dl.set_start (client_size.upper)
				pt.set_xy (0, client_size.upper.y)
				dl.set_final (pt)

				!!clr.black
				gr.set_pen (device, clr)
				device.draw (dl)
			end

		end

feature -- Effect DIALOG

	finish is
		do
			when_finished
		end

	set_font (ft: FONT) is
		do
		ensure then
			unimplemented: false
		end
	
end -- class TOOLBAR
