-- Fenestra; version 0.80(beta); 1998-09-27

-- 
-- Copyright (c) 1998 Franck Arnaud, http://www.altsoft.demon.co.uk/
-- 
-- This package is distributed under the Eiffel Forum Freeware License, 
-- version 1 (see file forum.txt for full text). The main points are:
-- 
-- "Permission is hereby granted, without written agreement and without 
--  license or royalty fees, to use, copy, modify and/or distribute this 
--  package, provided that: copyright notices are retained unchanged [...]
-- 
--  Permission is hereby also granted [...] to distribute binary programs 
--  which depend on this package, provided that: if the binary program 
--  depends on a modified version of this package, you must publicly release 
--  the modified version of this package [...].
-- 
--  THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT WARRANTY [...]"
-- 

indexing
	cluster: Fenestra, Xcontrols
	description: "Child windows stacked on each other in a direction"
	interface: client

class VERTICAL_STACK_WINDOW

inherit
	CHILD_WINDOW
		redefine
			when_resized
		end

--	P_TEXT_OBJECT

creation
	make, make_horizontal, make_vertical

feature {CREATION_NONE} -- Creation

	make, make_horizontal (p: like parent) is
			-- Create horizontal split window.
		do
			is_horizontal := true
			init (p)
		end

	make_vertical (p: like parent) is
			-- Create vertical split window.
		do
			is_horizontal := false
			init (p)
		end

feature {NONE} -- Attributes
	
	first, last, variable: CHILD_WINDOW
	is_horizontal: BOOLEAN

feature {NONE} -- Initialise
	
	init (p: like parent) is 
		do
			make_child (p)
			set_border (false)
		end

feature -- Setup
	
	set_first (win: CHILD_WINDOW) is
			-- Set right or top fixed size child window.
		require
			ok: win /= Void
			child: win.parent = Current
		do
			first := win
			when_resized
		ensure
			keep_reference: first = win
		end

	set_last (win: CHILD_WINDOW) is
			-- Set left or bottom fixed size child window.
		require
			ok: win /= Void
			child: win.parent = Current
		do
			last := win
			when_resized
		ensure
			keep_reference: last = win
		end

	set_variable (win: CHILD_WINDOW) is
			-- Variable size child window in the remaining space
			-- in the middle.
		require
			ok: win /= Void
			child: win.parent = Current
		do
			variable := win
			when_resized
		ensure
			keep_reference: variable = win
		end

feature {PUBLIC_NONE}
	
	when_resized is
			-- Window has been resized.
		local
			tmps, cs: RECTANGLE
			up, down, tmp_pt: POINT
			middle, actual: INTEGER
		do
			cs := client_size
			!! up.make
			!! down.make
			
			if not cs.is_thin then
			
				if first /= Void then
					tmps := first.size
					if tmps.height > cs.upper.y then -- enough space?
						first.hide
						-- up is (0, 0)
					else
						up.set_y (tmps.height)
						!! tmp_pt.make
						tmps.translate_lower (tmp_pt)
						tmps.set_width (cs.width)
						first.set_size (tmps)
						if not first.is_visible then first.show end
					end
				end

				-- default
				down.set_x (cs.upper.x)
				down.set_y (cs.upper.y)

				if last /= Void then
					tmps := last.size
					if tmps.height > (cs.upper.y - up.y) then -- enough space?
						last.hide
					else
						down.set_y (cs.upper.y - tmps.height + 1)
						!! tmp_pt.make_coord (0, down.y)
						tmps.translate_lower (tmp_pt)
						tmps.set_width (cs.width)
						last.set_size (tmps)
						if not last.is_visible then last.show end
					end
				end

				if variable /= Void then
					if down.y < up.y then -- nothing left
						variable.hide
					else
						set_and_show (variable, up, down)
					end
				end
			end -- has first & second
		end

feature {NONE} -- Implementation
	
	set_and_show (cw: CHILD_WINDOW; one, two: POINT) is
			-- Set window size and make it visible if necessary.
		require
			ok: cw /= Void
			one_not_void: one /= Void
			two_not_void: two /= Void
		local
			r: RECTANGLE
		do
			!!r.make
			r.set_points (one, two)

			cw.set_size (r)
			
			if not cw.is_visible then
				cw.show
			end
		end

end

